@@ -1,8 +1,8 @@
 /****************************************************
-* unit:    strings           release 0.20           *
+* unit:    strings           release 0.22           *
 * purpose: general manipulation with text strings   *
 * Licency: GPL or LGPL                              *
-* Copyright: (c) 1998-2019 Jaroslav Fojtik          *
+* Copyright: (c) 1998-2020 Jaroslav Fojtik          *
 *****************************************************/
 #include <stdlib.h>
 #include <stdio.h>
@@ -12,7 +12,7 @@
  #include <iostream.h>
 #endif
 
-#include <stringa.h>
+#include "stringa.h"
 
 
 #define ALLIGN_ALLOC 2
@@ -147,6 +147,20 @@ string *ps = &s;	// Workaround failure to access members
 }
 
 
+/// Create a new string and steal the content of the donor.
+/// Use pointer is tricky. You have function say:
+/// temp_string GetString(void)
+///......
+/// string result_fast(&GetString());	// This will extract contents
+/// string result_slow(GetString());	// This will duplicate temporary object
+string::string(temp_string *s)
+{
+string *ps = s;		// Workaround failure to access members
+ ch=ps->ch; size=ps->size; maxlen=ps->maxlen;
+ ps->ch=NULL; ps->size=ps->maxlen=0;
+}
+
+
 /// Allocates 'i+1' bytes and copy maximally i characters from str.
 string::string(const char *str, int i)
 {
@@ -851,20 +865,22 @@ int string::printf(const char* format, ...)
 #else
   size = vsprintf(NULL, format, paramList);
 #endif
+  va_end(paramList);
 
-  if(size)
+  if(size>0)
     {
-    resize(size);
+    resize(size+1);
+    va_start(paramList, format);
 #ifdef _HAVE_VSNPRINTF
     size = vsnprintf(ch, maxlen, format, paramList);
 #else
     size = vsprintf(ch, format, paramList);
 #endif
+    va_end(paramList);
     }
   else
     erase();
 
-  va_end(paramList);
   return size;
 }
 
@@ -878,19 +894,21 @@ int string::cat_printf(const char* format, ...)
 #else
   int size2 = vsprintf(NULL, format, paramList);
 #endif
+  va_end(paramList);
 
-  if(size2)
+  if(size2>0)
     {
     resize(size+size2+1);	// Keep one BYTE place for trailing 0.
+    va_start(paramList, format);
 #ifdef _HAVE_VSNPRINTF
     size2 = vsnprintf(ch+size, size2+1, format, paramList);
 #else
     size2 = vsprintf(ch+size, format, paramList);
 #endif
+    va_end(paramList);
     size += size2;
     }
 
-  va_end(paramList);
   return size2;
 }
 
