@@ -14,7 +14,6 @@
 #define PSS_FillColor 8
 
 
-void PS_Attr(string & PSData, PS_State *PSS);
 //void Polygon2PS(const float *Points, int Count, string & PSData, PS_State *PSS, 
 //                bool Outline=true, float ScaleFactor=WPGu2PSu(1));
 void StoreText2PS(string & PSData, string & Text, PS_State *PSS, WORD RotAngle,
@@ -22,10 +21,8 @@ void StoreText2PS(string & PSData, string & Text, PS_State *PSS, WORD RotAngle,
 		float FontSize);
 void Font2PS(string & Text, float FontSize, const char *FontName="Times-Roman");
 
-void FillObjectPS(float MinPolyX, float MaxPolyX, float MinPolyY, float MaxPolyY, string & PSData, PS_State *PSS);
-
 class WPG2_Transform;
 class TconvertedPass1;
-float *LoadPoints(TconvertedPass1 *cq, int n, FloatBBox &bbx, WPG2_Transform *TRx=NULL);
+float *LoadPoints(TconvertedPass1 *cq, int n, FloatBBox &bbx, WPG2_Transform *TRx=NULL,bool Precision=false);
 
 #endif // __IMAGES_H__
\ No newline at end of file
