@@ -1,16 +1,16 @@
 /***************************************************************************
- * Unit:    raster            release 0.18                                 *
+ * Unit:    raster            release 0.19                                 *
  * Purpose: Prototypes for general manipulation n dimensional matrices     *
  * Modul:   raster.cc                                                      *
  * Licency: GPL or LGPL                                                    *
- * Copyright: (c) 1998-2019 Jaroslav Fojtik                                *
+ * Copyright: (c) 1998-2020 Jaroslav Fojtik                                *
  ***************************************************************************/
 #ifndef __Rasters_h
 #define __Rasters_h
 
 #include "typedfs.h"
 
-#define RASTER_VERSION (0x100*0 | 18)
+#define RASTER_VERSION (0x100*0 | 19)
 
 #ifndef No_Memory
  #define No_Memory    0x1
@@ -412,17 +412,29 @@ typedef enum
 } IMAGE_TYPE;
 
 
+typedef enum
+{
+  EPS_DrawElipse =   1,
+  EPS_colorimage =   2,
+  EPS_rlecmapimage = 4,
+  EPS_accentshow  =  8,
+  EPS_ACCENTSHOW  = 16,
+} EPS_COMPONENTS;
+
+
+class VectorImage;
+
 class Image
 	{
 public: Raster2DAbstract *Raster;
 	APalette *Palette;
 	Image *Next;
 	float x,y,dx,dy,RotAngle;
-	char *Comment;
+	VectorImage *VecImage;
 
-	Image(void): Comment(NULL)  {RotAngle=x=y=dx=dy=0;Raster=NULL;Next=NULL;Palette=NULL;};
+	Image(void): VecImage(NULL)  {RotAngle=x=y=dx=dy=0;Raster=NULL;Next=NULL;Palette=NULL;};
 	Image(const Image & I, bool AllFrames=true);
-	Image(Raster2DAbstract *NewRaster): Comment(NULL) {x=y=dx=dy=0;Next=NULL;Palette=NULL;if((Raster=NewRaster)!=NULL) Raster->UsageCount++;};
+	Image(Raster2DAbstract *NewRaster): VecImage(NULL) {x=y=dx=dy=0;Next=NULL;Palette=NULL;if((Raster=NewRaster)!=NULL) Raster->UsageCount++;};
 	~Image(void) {Erase();};
 	IMAGE_TYPE ImageType(void) const;   // 0-none, 1-gray, 2-palette, 3-true color
 
@@ -438,6 +450,7 @@ public: Raster2DAbstract *Raster;
 	void Erase(void);
 	void AttachRaster(Raster2DAbstract *NewRaster);
         void AttachPalette(APalette *NewPalette);
+        void AttachVecImg(VectorImage *NewVecImg);
 	};
 
 
