@@ -3,9 +3,10 @@
  * function:    Object set for handling vector images.                        *
  * modul:       vec_image.cc                                                  *
  * licency:     GPL or LGPL                                                   *
- * Copyright: (c) 2018-2019 Jaroslav Fojtik                                   *
+ * Copyright: (c) 2018-2020 Jaroslav Fojtik                                   *
  ******************************************************************************/
 #include <stdlib.h>
+#include <stdio.h>
 #include <math.h>
 
 #include "vecimage.h"
@@ -89,6 +90,7 @@ vecFont::vecFont()
   ConvertCpg = NULL;
   Weight = 0;
   Italic = 0;
+  FontOrientation10 = 0;
   memset(&TextColor,0,sizeof(TextColor));
 }
 
@@ -100,6 +102,7 @@ vecFont::vecFont(const PS_State &PSS)
   FontSizeW = PSS.FontSizeW;
   Weight = PSS.FontWeight;
   Italic = PSS.FontItallic;
+  FontOrientation10 = PSS.FontOrientation10;
   memcpy(&TextColor, &PSS.TextColor, sizeof(TextColor));
 }
 
@@ -124,6 +127,10 @@ void vecFont::prepExport(PS_State *PSS) const
     {
       PSS->FontItallic = Italic;
     }
+    if(FontOrientation10 != PSS->FontOrientation10)
+    {
+      PSS->FontOrientation10 = FontOrientation10;
+    }
   }
 }
 
@@ -193,9 +200,9 @@ VectorEllipse::~VectorEllipse()
 }
 
 
-string VectorEllipse::Export2EPS(PS_State *PSS) const
+temp_string VectorEllipse::Export2EPS(PS_State *PSS) const
 {
-string str;
+temp_string str;
 
   if(PSS != NULL)
   {
@@ -246,7 +253,7 @@ void VectorEllipse::Transform(const AbstractTransformXY &Tx)
 /////////////////////////////////////////////////////////////
 
 
-VectorRectangle::VectorRectangle(float iniBottomRect, float iniTopRect, float iniRightRect, float iniLeftRect)
+VectorRectangle::VectorRectangle(float iniBottomRect, float iniTopRect, float iniLeftRect, float iniRightRect)
 {
  Tx = NULL;
  BottomRect = iniBottomRect;
@@ -263,9 +270,9 @@ VectorRectangle::~VectorRectangle()
 }
 
 
-string VectorRectangle::Export2EPS(PS_State *PSS) const
+temp_string VectorRectangle::Export2EPS(PS_State *PSS) const
 {
-string str;
+temp_string str;
 
   if(PSS != NULL)
   {
@@ -332,7 +339,7 @@ void VectorRectangleArc::Transform(const AbstractTransformXY &Tx)
 }
 
 
-string VectorRectangleArc::Export2EPS(PS_State *PSS) const
+temp_string VectorRectangleArc::Export2EPS(PS_State *PSS) const
 {
 float Ty = 1;
 bool ContextSaved = false;
@@ -340,7 +347,7 @@ bool ContextSaved = false;
   if(fabs(Hradius)<1e-5 || fabs(Vradius)<1e-5)
       return VectorRectangle::Export2EPS(PSS);
 
-  string PSData;
+  temp_string PSData;
 
   if(PSS != NULL)
   {
@@ -401,9 +408,9 @@ VectorPolygon::~VectorPolygon()
 }
 
 
-string VectorPolygon::Export2EPS(PS_State *PSS) const
+temp_string VectorPolygon::Export2EPS(PS_State *PSS) const
 {
-string str;
+temp_string str;
   if(Points==NULL || CountPoints<=0) return str;
 
   if(PSS != NULL)
@@ -450,6 +457,19 @@ void VectorPolygon::Transform(const AbstractTransformXY &Tx)
 
 /////////////////////////////////////////////////////////////
 
+VectorLine::VectorLine(int iniPointCount)
+{
+  Close = false;
+  if(iniPointCount<=0)
+  {
+    Points = NULL;
+    CountPoints = 0;
+    return;
+  }
+  Points = (float*)malloc(sizeof(float)*4);
+  CountPoints = iniPointCount;
+}
+
 
 VectorLine::VectorLine(float *iniPoints, int iniPointCount)
 {
@@ -470,9 +490,9 @@ VectorLine::~VectorLine()
 }
 
 
-string VectorLine::Export2EPS(PS_State *PSS) const
+temp_string VectorLine::Export2EPS(PS_State *PSS) const
 {
-string str;
+temp_string str;
 	// There must be at least two points for the line.
   if(Points==NULL || CountPoints<=1) return str;
 
@@ -511,61 +531,75 @@ void VectorLine::Transform(const AbstractTransformXY &Tx)
 /////////////////////////////////////////////////////////////
 
 
+void PsBlob::FeaturesEPS(unsigned char & Feature) const
+{
+  if(Blob==NULL) return;
+  if(StrStr(Blob,"DrawEllipse")!=NULL) Feature |= EPS_DrawElipse;
+  if(StrStr(Blob,"accentshow")!=NULL)  Feature |= EPS_accentshow;
+  if(StrStr(Blob,"ACCENTSHOW")!=NULL)  Feature |= EPS_ACCENTSHOW;
+}
+
+
+/////////////////////////////////////////////////////////////
+
+
 VectorList::VectorList(void)
 {
-  Objects = NULL;
+  FirstObject = LastObject = NULL;
   VectorObjects =0;
 }
 
 
 VectorList::~VectorList()
 {
-  if(Objects!=NULL)
+VectorObject *IterObject = FirstObject;
+
+  while(IterObject!=NULL)
   {
-    for(int i=0; i<VectorObjects; i++)
-    {
-      if(Objects[i] != NULL)
-      {
-        delete(Objects[i]);
-        Objects[i] = NULL;
-      }
-    }
-    free(Objects);
-    Objects = NULL;
-  }  
+    VectorObject *ErasedObject = IterObject;
+    IterObject = IterObject->NextObject;
+    delete(ErasedObject);
+  }
+  
+  FirstObject = LastObject = NULL;
   VectorObjects = 0;
 }
 
 
-string VectorList::Export2EPS(PS_State *PSS) const
+temp_string VectorList::Export2EPS(PS_State *PSS) const
 {
-string str;
+temp_string str;
+VectorObject *IterObject = FirstObject;
 
-  if(Objects!=NULL && VectorObjects>0)
-  {    
-    for(int i=0; i<VectorObjects; i++)
-    {
-      if(Objects[i]!=NULL)
-      {
-        str += Objects[i]->Export2EPS(PSS);
-      }
-    }
+  while(IterObject!=NULL)
+  {
+    str += IterObject->Export2EPS(PSS);
+    IterObject = IterObject->NextObject;
   }
   return str;
 }
 
 
+void VectorList::FeaturesEPS(unsigned char & Feature) const
+{
+VectorObject *IterObject = FirstObject;
+
+  while(IterObject!=NULL)
+  {
+    IterObject->FeaturesEPS(Feature);
+    IterObject = IterObject->NextObject;
+  }
+
+}
+
+
 void VectorList::Transform(const AbstractTransformXY &Tx)
 {
-  if(Objects!=NULL && VectorObjects>0)
-  {    
-    for(int i=0; i<VectorObjects; i++)
-    {
-      if(Objects[i]!=NULL)
-      {
-        Objects[i]->Transform(Tx);
-      }
-    }
+VectorObject *IterObject = FirstObject;
+  while(IterObject!=NULL)
+  {
+    IterObject->Transform(Tx);
+    IterObject = IterObject->NextObject;
   }
 }
 
@@ -574,18 +608,47 @@ void VectorList::AddObject(VectorObject *NewObj)
 {
   if(NewObj==NULL || NewObj==this) return;
 
-  if(Objects==NULL || VectorObjects<=0)
+  if(FirstObject==NULL)
   {
-    Objects = (VectorObject**)malloc(sizeof(VectorObject*));
-    if(Objects==NULL) return;
-    Objects[0] = NewObj;    
-    VectorObjects = 1;
-    return;
+    FirstObject = LastObject = NewObj;
+  }
+  else
+  {
+   LastObject->NextObject = NewObj;
+   LastObject = NewObj;
   }
-
   VectorObjects++;
-  Objects = (VectorObject**)realloc(Objects, (VectorObjects)*sizeof(VectorObject*));
-  Objects[VectorObjects-1] = NewObj;
+}
+
+
+void VectorList::Export2EPS(FILE *F, PS_State *PSS) const
+{
+VectorObject *IterObject = FirstObject;
+  if(F==NULL) return;
+  while(IterObject!=NULL)
+  {
+    fputs(IterObject->Export2EPS(PSS)(), F);
+    IterObject = IterObject->NextObject;
+  }
+}
+
+/////////////////////////////////////////////////////////////
+
+
+VectorImage::VectorImage(VectorList &VecList, const PS_State & NewPSS)
+{
+  PSS = NewPSS;
+  UsageCount = 0;
+  FirstObject = VecList.FirstObject;	VecList.FirstObject = NULL;
+  LastObject = VecList.LastObject;	VecList.LastObject = NULL;
+  VectorObjects = VecList.VectorObjects; VecList.VectorObjects =0;
+}
+
+
+VectorImage::VectorImage()
+{
+  UsageCount=0;
+  memset(&PSS, 0, sizeof(PSS));
 }
 
 
@@ -619,7 +682,13 @@ static const char *GreekText[] =
      "$\\Omega$", "$\\omega$",		//W
      "$\\Xi$", "$\\xi$",		//X
      "$\\Psi$", "$\\psi$",		//Y					
-     "$\\ZETA$", "$\\zeta$"		//Z
+     "$\\ZETA$", "$\\zeta$",		//Z
+
+	// Math symbols that are using Greek alphabet
+     "$\\sum$",
+     "$\\int$",
+     "$\\rightarrow$",
+     "$\\leftarrow$"
      };
 
 static const char PsText[] = 
@@ -649,6 +718,11 @@ static const char PsText[] =
      'X', 'x',
      'Y', 'y',
      'Z', 'x',
+	// Math symbols that are using Greek alphabet
+     'S',
+     '\362',		// integral - https://www.sscnet.ucla.edu/soc/faculty/mcfarland/soc281/symbol.htm
+     '\256',		// right arrow
+     '\254'		// left arrow
     };
 
 
@@ -657,6 +731,8 @@ TextContainer::TextContainer()
 {
   Text = NULL;
   TextObjects = 0;
+  FontOrientation = 0;
+  TextColor.Red = TextColor.Green = TextColor.Blue = 0;		// Use black.
 }
 
 
@@ -679,9 +755,9 @@ TextContainer::~TextContainer()
 }
 
 
-string TextContainer::Export2EPS(PS_State *PSS) const
+temp_string TextContainer::Export2EPS(PS_State *PSS) const
 {
-string str;
+temp_string str;
 
   if(Text!=NULL && TextObjects>0)
   {
@@ -695,6 +771,9 @@ string str;
       PS_Attr(str,PSS);
     }
 
+    if(fabs(FontOrientation) > 0.01)    
+        str += "\ngsave";    
+
     for(int i=0; i<TextObjects; i++)
     {
       if(Text[i]!=NULL)
@@ -709,13 +788,19 @@ string str;
             Text[i]->TargetFont(), Text[i]->size*2.66f);
 
         if(i==0)
-            str.cat_printf("\nnewpath %2.2f %2.2f moveto", PosX, PosY);
+        {
+          str.cat_printf("\nnewpath %2.2f %2.2f moveto", PosX, PosY);
+          if(fabs(FontOrientation) > 0.01)          
+	      str.cat_printf("\n%g rotate", FontOrientation);          
+        }
 
         str += "\n(";
         str += Text[i]->contents + ')';
       }      
     }
     str += "\nshow";
+    if(fabs(FontOrientation) > 0.01)    
+        str += "\ngrestore";    
   }
 
 return str;
@@ -775,7 +860,13 @@ TextObject *pTobj;
   }
 
   TextObjects++;
-  Text = (TextObject**)realloc(Text, (TextObjects)*sizeof(TextObject*));
+  TextObject** tmp = (TextObject**)realloc(Text, (TextObjects)*sizeof(TextObject*));
+  if(tmp==NULL) 
+    {	// @TODO - report a memory problem.
+    TextObjects--;
+    return;
+    }
+  Text = tmp;
   pTobj = Text[TextObjects-1] = new TextObject;
   pTobj->TargetFont = font;
   pTobj->contents = contents;
@@ -795,6 +886,21 @@ bool TextContainer::isEmpty(void) const
   return true;
 }
 
+
+static void CheckAccent(unsigned char & Feature, const char *PS_text)
+{
+  if(PS_text==NULL) return;
+  if(StrStr(PS_text,"accentshow")!=NULL)  Feature |= EPS_accentshow;
+  if(StrStr(PS_text,"ACCENTSHOW")!=NULL)  Feature |= EPS_ACCENTSHOW;
+}
+
+void TextContainer::FeaturesEPS(unsigned char & Feature) const
+{
+  CheckAccent(Feature, Export2EPS());
+}
+
+
+
 /////////////////////////////////////////////////////////////
 
 #include "typedfs.h"
@@ -805,8 +911,8 @@ void Color2PS(string & Text, const RGB_Record & TextColor);
 
 
 /** Transfer accent to PostSctipt text.
- * @param[in,out] s		String with text.
- * @param[in]	TexAccent	Accent prefix in LeTex.
+ * @param[in,out] s		String with text to be fixed.
+ * @param[in]	TexAccent	Accent prefix in LaTeX.
  * @param[in]   PsAccent	Accent equivalent in PS.*/
 static void FixPsAccent(string & s, const char *TexAccent, const char *PsAccent)
 {
@@ -936,7 +1042,7 @@ float AdjY;
   //AdjY = UpRightY - AdjY;
   PSData.cat_printf("\nnewpath %2.2f %2.2f moveto\n", LowLeftX,AdjY);
   Color2PS(PSData,PSS->TextColor);
-  PSS->dirty |= PSS_LineColor;
+  PSS->dirty |= PSS_LineColor; 
 
   FixPsAccent(Text,"\\'{","(\\302)");	// acute
   FixPsAccent(Text,"\\v{","(\\317)");	// charon
@@ -978,3 +1084,155 @@ float AdjY;
 
   if(RotAngle>0) PSData+="\ngrestore";
 }
+
+
+/// Emit Postscript command for line type change.
+void PS_Attr(string & PSData, PS_State *PSS)
+{
+ if(PSS->dirty & PSS_LineColor)
+   {
+   PSData.cat_printf("\n%0.2g %0.2g %0.2g setrgbcolor",
+	(float)PSS->LineColor.Red/256.0,
+	(float)PSS->LineColor.Green/256.0,
+	(float)PSS->LineColor.Blue/256.0);
+   PSS->dirty &= ~PSS_LineColor;
+   }
+ if(PSS->dirty & PSS_LineStyle)
+   {
+   switch(PSS->LineStyle)
+	{
+	case 2:PSData+="\n[12 4]0"; break;
+	case 3:PSData+="\n[2 2]0"; break;
+	case 4:PSData+="\n[8 3 2 3]0"; break;
+	case 5:PSData+="\n[8 8]0"; break;
+	case 6:PSData+="\n[6 2 2 2 2 2]0"; break;
+	case 7:PSData+="\n[4 4]0"; break;
+	case 8:PSData+="\n[8 2 3 2]0"; break;
+	case 9:PSData+="\n[14 2]0"; break;
+	case 10:PSData+="\n[1 1]0"; break;
+	case 11:PSData+="\n[1 3]0"; break;
+	case 12:PSData+="\n[3 5]0"; break;
+	case 13:PSData+="\n[8 2 4 2]0"; break;
+	case 14:PSData+="\n[2 2 10 2]0"; break;
+	case 15:PSData+="\n[10 2 2 2]0"; break;
+	case 16:PSData+="\n[10 2 2 2 2 2]0"; break;	//type No13 in wpg2
+	case 17:PSData+="\n[12 4 12 4 4 4]0"; break;	//type No14 in wpg2
+	case 18:PSData+="\n[12 2 12 2 3 2 3 2]0"; break;//type No15 in wpg2
+	case 19:PSData+="\n[1 2 1 2 1 5]0"; break;	//type No7 in wpg2
+	case 1:
+	//case 0: //no line
+	default:PSData+="\n[]0";
+	}
+   PSData+=" setdash";
+   PSS->dirty &= ~PSS_LineStyle;
+   }
+ if(PSS->dirty & PSS_LineWidth)
+  {
+  PSData.cat_printf("\n%2.2f setlinewidth",PSS->LineWidth);
+  PSS->dirty &= ~PSS_LineWidth;
+  }
+}
+
+
+void FillObjectPS(float MinPolyX, float MaxPolyX, float MinPolyY, float MaxPolyY, string & PSData, PS_State *PSS)
+{
+string tmp, color;
+float Distance;
+
+  if(PSS==NULL)
+  {
+    PSData += "\nstroke";		//draw frame line
+    return;
+  }
+
+  if(PSS->FirstTimeFix &&
+     PSS->FillColor.Red == PSS->FillBackground.Red &&
+     PSS->FillColor.Green == PSS->FillBackground.Green &&
+     PSS->FillColor.Blue == PSS->FillBackground.Blue)
+    {
+    PSS->FillColor.Red = ~PSS->FillColor.Red;
+    PSS->FillColor.Green = ~PSS->FillColor.Green;
+    PSS->FillColor.Blue = ~PSS->FillColor.Blue;
+    PSS->FirstTimeFix = false;
+    }
+
+  color.printf("\n%0.2g %0.2g %0.2g setrgbcolor",
+		(float)PSS->FillColor.Red/256.0,
+	 	(float)PSS->FillColor.Green/256.0,
+		(float)PSS->FillColor.Blue/256.0);
+
+  switch(PSS->FillPattern)
+		{
+		case 1:if(PSS->LineStyle!=0)
+			 PSData+="\ngsave";
+		       PSData+=color;
+		       PSData+="\nfill";
+		       if(PSS->LineStyle!=0)
+			 PSData+=" grestore";
+		       else
+			 PSS->dirty |= PSS_LineColor;
+		       break;
+		case 2:Distance = 2.8;
+		       goto FINALDiagonal;
+		case 3:Distance = 5.7;
+		       goto FINALDiagonal;
+		case 4:Distance = 11.3;
+FINALDiagonal:	       tmp.printf("\n%2.2f %2.1f %2.2f {%2.2f moveto %2.2f %2.2f rlineto}",
+			       MinPolyX-(MaxPolyY-MinPolyY),Distance,MaxPolyX,MinPolyY,
+			       MaxPolyY-MinPolyY, MaxPolyY-MinPolyY  );
+		       goto FINALLY;
+		case 5:Distance=2.8;
+		       goto FINALCross;
+		case 6:Distance=5.7;
+		       goto FINALCross;
+		case 7:Distance=11.3;
+FINALCross:	       tmp.printf("\n%2.2f %2.1f %2.2f {%2.2f moveto %2.2f %2.2f rlineto}for",
+			       MinPolyX-(MaxPolyY-MinPolyY),Distance,MaxPolyX,MinPolyY,
+			       MaxPolyY-MinPolyY, MaxPolyY-MinPolyY );
+			PSData+='\n';
+			if(PSS->LineStyle!=0) PSData+="gsave";
+			PSData+=" clip newpath 0 setlinewidth\n";
+			PSData+=color;
+			PSData+=tmp;
+			tmp.printf("\n%2.2f %2.1f %2.2f {%2.2f moveto %2.2f %2.2f rlineto}",
+			       MinPolyX,Distance,(MaxPolyX+(MaxPolyY-MinPolyY)),MinPolyY,
+			       -(MaxPolyY-MinPolyY), MaxPolyY-MinPolyY  );
+			goto FINALLY2;
+		case 8: Distance=2;
+			goto FINALVertical;
+		case 9: Distance=4;
+			goto FINALVertical;
+		case 10:Distance=8;
+FINALVertical:		tmp.printf("\n%2.2f %2.1f %2.2f {%2.2f moveto 0 %2.2f rlineto}",
+			       MinPolyX,Distance,MaxPolyX,MinPolyY,
+			       (MaxPolyY-MinPolyY)  );
+FINALLY:		PSData+='\n';
+			if(PSS->LineStyle!=0) PSData+="gsave";
+			PSData+=" clip newpath 0 setlinewidth\n";
+			PSData+=color;
+FINALLY2:		PSData+=tmp;
+			PSData+="for\nstroke";
+			if(PSS->LineStyle!=0) PSData+=" grestore";
+			break;
+				case 11:Distance=16;
+FINALDot:		tmp.printf("\n[0.5 %2.1f] 0 setdash\n %2.2f %2.1f %2.2f {%2.2f moveto 0 %2.2f rlineto %2.2f %2.2f rmoveto 0 %2.2f rlineto}",
+			       Distance-0.5,
+			       MinPolyX,Distance,MaxPolyX,
+			       MinPolyY,
+			       MaxPolyY-MinPolyY,
+			       Distance/2.0,-Distance/2.0 - (MaxPolyY-MinPolyY),
+			       (MaxPolyY-MinPolyY)+Distance/2.0  );
+			goto FINALLY;
+		case 12:Distance=8;
+			goto FINALDot;
+		case 13:Distance=4;
+			goto FINALDot;
+		case 14:Distance=2;
+			goto FINALDot;
+		case 15:Distance=1;
+			goto FINALDot;
+		}
+
+  if(PSS->LineStyle!=0)
+    PSData+="\nstroke";		//draw frame line
+}
