@@ -11,11 +11,12 @@
 #include <stdlib.h>
 #include <ctype.h>
 
-#include <sets.h>
-#include <stringa.h>
-#include <lists.h>
-#include "wp2latex.h"
+#include "sets.h"
+#include "stringa.h"
+#include "lists.h"
+#include "struct.h"
 
+#include "wp2latex.h"
 #include "cp_lib/cptran.h"
 
 
@@ -116,7 +117,7 @@ void TconvertedPass1_WP4::Comment4(DWORD & NewPos, WORD & w)
   Old_char_on_line = char_on_line;
   flag = CharsOnly;
   recursion++;
-  InitAttr(attr);		//Turn all attributes in the comment off
+  attr.InitAttr();		//Turn all attributes in the comment off
 
   fseek(wpd, 4, SEEK_CUR);
 
@@ -179,7 +180,7 @@ void TconvertedPass1_WP4::FootEndNote(DWORD & NewPos, WORD & w)
   OldAttr = attr;
   flag = HeaderText;
   recursion++;
-  InitAttr(attr);		//Turn all attributes in the note off
+  attr.InitAttr();		//Turn all attributes in the note off
 
   NewPos = ftell(wpd);
   fread(&NoteType, 1, 1, wpd);
@@ -241,7 +242,7 @@ void TconvertedPass1_WP4::FootNoteObsolette(DWORD & NewPos, WORD & w)
   OldAttr = attr;
   flag = CharsOnly;
   recursion++;
-  InitAttr(attr);		//Turn all attributes in the note off
+  attr.InitAttr();		//Turn all attributes in the note off
 
   fseek(wpd, 5L, SEEK_CUR);
   fprintf(strip, "\\footnote{");
@@ -320,7 +321,7 @@ void TconvertedPass1_WP4::Header_Footer4(DWORD & NewPos, WORD & w)
 
 // Start a conversion of a Header or Footer
   LogLevel = SaveLLevel;
-  InitAttr(attr);		//Turn all attributes in the header/footer off
+  attr.InitAttr();		//Turn all attributes in the header/footer off
   recursion++;
 
   line_term = 's';    	//Soft return
@@ -429,7 +430,7 @@ void TconvertedPass1_WP4::MakeIndex4(DWORD & NewPos)
   Old_char_on_line = char_on_line;
   flag = CharsOnly;
   recursion++;
-  InitAttr(attr);		//Turn all attributes in the index off
+  attr.InitAttr();		//Turn all attributes in the index off
 
 
   fprintf(strip, " \\index{");
