@@ -24,18 +24,28 @@ string CutFileName(const char *FullFilename); //from wp2lfuti.cc
 int SavePictureEPS(const char *Name,const Image &Img); //froom ras_img.cc
 
 
-#define AbiwordVersion "0.9"
+#define AbiwordVersion "0.10"
 
 
 /*Register translators here*/
 class TconvertedPass1_Abi: public TconvertedPass1_XML
-     {
+{
 public:
      virtual int Convert_first_pass(void);
-     };
+
+     void CharPropAbi(void);
+     void FieldAbi(void);
+     void FrameAbi(void);
+     void ImageAbi(void);
+     void ParPropAbi(void);
+     void ProcessKeyABI(void);
+     void ProcessDataAbi(void);
+     void SkipMetaAbi(void);
+     void TableABI(void);
+};
+
 TconvertedPass1 *Factory_Abi(void) {return new TconvertedPass1_Abi;}
 FFormatTranslator FormatAbiword("abiword",Factory_Abi);
-static void ProcessKeyABI(TconvertedPass1_XML *cq);
 
 
 enum AbiKeywords
@@ -57,7 +67,7 @@ enum AbiKeywords
 
 
 /** Break property string into double list property, value. */
-static void ParseProps(char *PropStr,doublelist & d)
+static void ParseProps(char *PropStr, doublelist & d)
 {
 char *BegProp,*EndProp,*BgPropVal,*EndPropVal;
 char c1,c2;
@@ -91,40 +101,39 @@ char c1,c2;
 }
 
 
-
-static void ParPropAbi(TconvertedPass1_XML *cq)
+void TconvertedPass1_Abi::ParPropAbi(void)
 {
 #ifdef DEBUG
-  fprintf(cq->log,"\n#ParPropAbi() ");fflush(cq->log);
+  fprintf(log,"\n#ParPropAbi() ");fflush(log);
 #endif
 int i;
 char *Props=NULL;
 doublelist d;
 
-  if((i=cq->TAG_Args IN "props")>=0)
+  if((i=TAG_Args IN "props")>=0)
      {
-     Props=cq->TAG_Args.Member(i,1);
+     Props = TAG_Args.Member(i,1);
      ParseProps(Props,d);
      if((i=d IN "text-align")>=0)
 	{			    /*0x80-Left, 0x83-Right, 0x82-Center, 0x81-Full */
 	Props=d.Member(i,1);
-	if(!StrCmp(Props,"center")) Justification(cq, 0x82);
-	if(!StrCmp(Props,"right"))  Justification(cq, 0x83);
-	if(!StrCmp(Props,"left"))   Justification(cq, 0x80);
-	if(!StrCmp(Props,"justify"))Justification(cq, 0x81);
+	if(!StrCmp(Props,"center")) Justification(this, 0x82);
+	if(!StrCmp(Props,"right"))  Justification(this, 0x83);
+	if(!StrCmp(Props,"left"))   Justification(this, 0x80);
+	if(!StrCmp(Props,"justify"))Justification(this, 0x81);
 	}
      }
-  if(cq->char_on_line) HardReturn(cq);  //Paragraph on
-  SoftReturn(cq);
+  if(char_on_line) HardReturn(this);  //Paragraph on
+  SoftReturn(this);
 
 return;
 }
 
 
-static void CharPropAbi(TconvertedPass1_XML *cq)
+void TconvertedPass1_Abi::CharPropAbi(void)
 {
 #ifdef DEBUG
-  fprintf(cq->log,"\n#CharPropAbi() ");fflush(cq->log);
+  fprintf(log,"\n#CharPropAbi() ");fflush(log);
 #endif
 int i;
 char *Props=NULL;
@@ -132,10 +141,10 @@ string s;
 doublelist d;
 attribute NewAttr;
 
-  InitAttr(NewAttr);
-  if((i=cq->TAG_Args IN "props")>=0)
+  //InitAttr(NewAttr);
+  if((i=TAG_Args IN "props")>=0)
      {
-     Props=cq->TAG_Args.Member(i,1);
+     Props=TAG_Args.Member(i,1);
      ParseProps(Props,d);
 
      if((i=d IN "color")>=0)
@@ -166,34 +175,34 @@ attribute NewAttr;
 	}
      }
 
-  AttrFit(cq->attr,NewAttr,s);
-  if(s!="") fputs(s,cq->strip);
+  AttrFit(attr,NewAttr,s);
+  if(s!="") fputs(s,strip);
 
 return;
 }
 
 
-static void FieldAbi(TconvertedPass1_XML *cq)
+void TconvertedPass1_Abi::FieldAbi(void)
 {
 #ifdef DEBUG
-  fprintf(cq->log,"\n#FieldAbi() ");fflush(cq->log);
+  fprintf(log,"\n#FieldAbi() ");fflush(log);
 #endif
 int i;
-char *Type;
+const char *Type;
 
-  if((i=cq->TAG_Args IN "type")>=0)
+  if((i=TAG_Args IN "type")>=0)
      {
-     Type=cq->TAG_Args.Member(i,1);
+     Type=TAG_Args.Member(i,1);
      if(Type)
-	if(!strcmp(Type,"page_number")) PageNumber(cq);
+	if(!strcmp(Type,"page_number")) PageNumber(this);
      }
 }
 
 
-static void FrameAbi(TconvertedPass1_XML *cq)
+void TconvertedPass1_Abi::FrameAbi(void)
 {
 #ifdef DEBUG
-  fprintf(cq->log,"\n#FrameAbi() ");fflush(cq->log);
+  fprintf(log,"\n#FrameAbi() ");fflush(log);
 #endif
 TBox Box;
 int i;
@@ -201,20 +210,15 @@ const char *FileName = NULL;
 char *Props;
 doublelist d;
 
- if((i=cq->TAG_Args IN "strux-image-dataid")>=0)
+ if((i=TAG_Args IN "strux-image-dataid")>=0)
  {
-   Box.Width = -1; 		// Undefined, use default 100mm
-   Box.Image_type = 0;		// Image on disk, even when embedded. Would be extracted later.
-   Box.AnchorType = 0; 		// 0-Paragraph, 1-Page, 2-Character
-   Box.HorizontalPos = 2;	// 0-Left, 1-Right, 2-Center, 3-Full
-   Box.Image_size = 0;
-   Box.RotAngle = 0;
+   initBox(Box);   
 
-   FileName = cq->TAG_Args.Member(i,1);
+   FileName = TAG_Args.Member(i,1);
    if(FileName==NULL) FileName="dummy";
-   if((i=cq->TAG_Args IN "props")>=0)
+   if((i=TAG_Args IN "props")>=0)
      {
-     Props = cq->TAG_Args.Member(i,1);
+     Props = TAG_Args.Member(i,1);
      ParseProps(Props,d);
 
      if((i=d IN "frame-width")>=0)
@@ -225,23 +229,23 @@ doublelist d;
 
 	if(width>0)
 	  {
-	  if(strstr(w_props,"mm"))	{Box.Width = width;}
-	  else if(strstr(w_props,"cm")) {Box.Width = 10*width;}
-	  else if(strstr(w_props,"in")) {Box.Width = 25*width;} //inch
+	  if(strstr(w_props,"mm"))	{Box.Width = width/10;}
+	  else if(strstr(w_props,"cm")) {Box.Width = width;}
+	  else if(strstr(w_props,"in")) {Box.Width = 2.54*width;} //inch
           }
 	}
      }
 
-   ImageWP(cq, FileName, Box);
+   ImageWP(this, FileName, Box);
  }
 return;
 }
 
 
-static void ImageAbi(TconvertedPass1_XML *cq)
+void TconvertedPass1_Abi::ImageAbi(void)
 {
 #ifdef DEBUG
-  fprintf(cq->log,"\n#ImageAbi() ");fflush(cq->log);
+  fprintf(log,"\n#ImageAbi() ");fflush(log);
 #endif
 TBox Box;
 int i;
@@ -249,23 +253,30 @@ const char *FileName=NULL;
 char *Props;
 doublelist d;
 
-  Box.Width = -1;		// Undefined, use default 100mm
-  Box.Image_type=0;		// Image on disk
-  Box.AnchorType = 0; 		// 0-Paragraph, 1-Page, 2-Character
-  Box.HorizontalPos=2;		// 0-Left, 1-Right, 2-Center, 3-Full
-  Box.Image_size = 0;
-  Box.RotAngle = 0;
+  initBox(Box);
   
-  if((i=cq->TAG_Args IN "dataid")>=0)
-	FileName=cq->TAG_Args.Member(i,1);
-  if((i=cq->TAG_Args IN "props")>=0)
+  if((i=TAG_Args IN "dataid")>=0)
+	FileName = TAG_Args.Member(i,1);
+  if((i=TAG_Args IN "props")>=0)
      {
-     Props=cq->TAG_Args.Member(i,1);
+     Props = TAG_Args.Member(i,1);
      ParseProps(Props,d);
+     if((i=d IN "width")>=0)
+	{
+	const char *w_props = d.Member(i,1);
+	double width = atof(w_props);
+
+        if(width>0)
+	  {
+	  if(strstr(w_props,"mm"))	{Box.Width = width/10;}
+	  else if(strstr(w_props,"cm")) {Box.Width = width;}
+	  else if(strstr(w_props,"in")) {Box.Width = 2.54*width;} //inch
+          }	
+        }
      }
   if(FileName==NULL) FileName="dummy";
 
-  ImageWP(cq, FileName, Box);
+  ImageWP(this, FileName, Box);
 return;
 }
 
@@ -297,10 +308,10 @@ return;
 }
 
 
-static void ProcessDataAbi(TconvertedPass1_XML *cq)
+void TconvertedPass1_Abi::ProcessDataAbi(void)
 {
 #ifdef DEBUG
-  fprintf(cq->log,"\n#ProcessDataAbi() ");fflush(cq->log);
+  fprintf(log,"\n#ProcessDataAbi() ");fflush(log);
 #endif
 unsigned char OldFlag;
 FILE *f;
@@ -309,43 +320,43 @@ char data64;
 char cnt;
 string filename;
 
- OldFlag = cq->flag;
- cq->flag = Nothing;
+ OldFlag = flag;
+ flag = Nothing;
  
- if((cnt=cq->TAG_Args IN "name")>=0)
+ if((cnt=TAG_Args IN "name")>=0)
    {   
-   filename=cq->TAG_Args.Member(cnt,1);
+   filename = TAG_Args.Member(cnt,1);
    }
  if(length(filename)==0)
    filename=GetSomeImgName(".png"); //invent some filename if empty
 
  filename = MergePaths(OutputDir,RelativeFigDir)+CutFileName(filename)+".png";
 
- f = OpenWrChk(filename(),"wb",cq->err);
+ f = OpenWrChk(filename(),"wb",err);
 
  cnt=0;
  do {
-    cq->ReadXMLTag();
-    if(cq->by==0)
+    ReadXMLTag();
+    if(this->by==0)
       {
-      cq->subby = cq->TAG[0];
-      if(cq->subby==0 || cq->subby=='\n' || cq->subby=='\r' || cq->subby=='\t')
+      subby = TAG[0];
+      if(subby==0 || subby=='\n' || subby=='\r' || subby=='\t')
  	 continue;
 
-      if(cq->subby>='A' && cq->subby<='Z')
-        data64 = cq->subby - 'A';
-      else if(cq->subby>='a' && cq->subby<='z')
-        data64 = cq->subby - 'a' + 26;
-      else if(cq->subby>='0' && cq->subby<='9')
-        data64 = cq->subby - '0' + 2*26;
-      else if(cq->subby=='+') data64 = 62;
-      else if(cq->subby=='/') data64 = 63;
+      if(subby>='A' && subby<='Z')
+        data64 = subby - 'A';
+      else if(subby>='a' && subby<='z')
+        data64 = subby - 'a' + 26;
+      else if(subby>='0' && subby<='9')
+        data64 = subby - '0' + 2*26;
+      else if(subby=='+') data64 = 62;
+      else if(subby=='/') data64 = 63;
       else
         {
-        if (cq->err != NULL)
+        if(err != NULL)
  	  {
- 	  cq->perc.Hide();
-	  fprintf(cq->err, _("\nWarning: Invalid character '%c' (%d) inside base64 data will be ignored!"),cq->subby,cq->subby);
+ 	  perc.Hide();
+	  fprintf(err, _("\nWarning: Invalid character '%c' (%d) inside base64 data will be ignored!"),this->subby,this->subby);
 	  }
         data64 = 0;
         continue;
@@ -368,10 +379,10 @@ string filename;
      }
     else
       {
-      ProcessKeyABI(cq);
-      if(cq->by==ABI_DATA && cq->subby==1) break;
+      ProcessKeyABI();
+      if(this->by==ABI_DATA && this->subby==1) break;
       }
-    } while(!feof(cq->wpd));
+    } while(!feof(wpd));
 
  if(f)
     {
@@ -392,72 +403,72 @@ string filename;
       filename = MergePaths(OutputDir,RelativeFigDir)+CutFileName(filename)+".eps";
       if(SavePictureEPS(filename(),Img)<0)
         {
-        if(cq->err != NULL)
+        if(err != NULL)
 	  {
-	  cq->perc.Hide();
-	  fprintf(cq->err, _("\nError: Cannot save file: \"%s\"!"),filename());
+	  perc.Hide();
+	  fprintf(err, _("\nError: Cannot save file: \"%s\"!"),filename());
 	  }
         }
       }      
     }
 
-  cq->flag = OldFlag;
+  flag = OldFlag;
 }
 
 
-static void SkipMetaAbi(TconvertedPass1_XML *cq)
+void TconvertedPass1_Abi::SkipMetaAbi(void)
 {
 #ifdef DEBUG
-  fprintf(cq->log,"\n#SkipMetaAbi() ");fflush(cq->log);
+  fprintf(log,"\n#SkipMetaAbi() ");fflush(log);
 #endif
 unsigned char OldFlag;
 
-OldFlag = cq->flag;
-cq->flag = Nothing;
+ OldFlag = flag;
+ flag = Nothing;
 
-do {
-   cq->TAG = "";
-   ProcessKeyABI(cq);
-   if(cq->by==ABI_META && cq->subby==1) break;
-   if(cq->by==ABI_SECTMETA && cq->subby==1) break;
-   } while(!feof(cq->wpd));
+ do {
+    TAG = "";
+    ProcessKeyABI();
+    if(by==ABI_META && subby==1) break;
+    if(by==ABI_SECTMETA && subby==1) break;
+    } while(!feof(wpd));
 
-cq->flag = OldFlag;
+ flag = OldFlag;
 }
 
 
-static void TableABI(TconvertedPass1_XML *cq)
+void TconvertedPass1_Abi::TableABI(void)
 {
 #ifdef DEBUG
-  fprintf(cq->log,"\n#TableABI() ");fflush(cq->log);
+  fprintf(log,"\n#TableABI() ");fflush(log);
 #endif
 unsigned char OldFlag;
 char OldEnvir;
 long FilePos;
-int i,FieldCount=0;
+int i, FieldCount=0;
 char *Prop;
 char *FieldPos=NULL;
 char Alignment;
 doublelist PrpL;
 
-FilePos = ftell(cq->wpd);
-OldFlag = cq->flag;
-OldEnvir = cq->envir;
-cq->flag = Nothing;
-cq->recursion++;
+FilePos = ftell(wpd);
+OldFlag = flag;
+OldEnvir = envir;
+flag = Nothing;
+recursion++;
 
 do {
-   cq->TAG = "";
-   ProcessKeyABI(cq);
-   if(cq->by==ABI_TABLE)
+   TAG = "";
+   ProcessKeyABI();
+   if(by==ABI_TABLE)
      {
-     if(cq->subby == 0) break;			// <TABLE>
-     if(cq->subby == 1) break;			// </TABLE>
-     if(cq->subby == 2)				// <CELL>
+     if(subby == 0) break;			// <TABLE>
+     if(subby == 1) break;			// </TABLE>
+     if(subby == 2)				// <CELL>
        {
-       if((i=cq->TAG_Args IN "props")>=0)
+       if((i=TAG_Args IN "props")>=0)
           {
-          ParseProps(cq->TAG_Args.Member(i,1),PrpL);
+          ParseProps(TAG_Args.Member(i,1),PrpL);
           if((i=PrpL IN "top-attach")>=0)
 	    {
 	    Prop=PrpL.Member(i,1);
@@ -466,142 +477,142 @@ do {
 	    }
           }
        }
-     if(cq->subby == 3) 			// </CELL>
+     if(subby == 3) 			// </CELL>
 	{
 	FieldCount++;
 	}
      }
-   }while(!feof(cq->wpd));
+   }while(!feof(wpd));
 
 
   if(FieldCount>0)
     {
-    cq->line_term = 's';   /* Soft return */
-    if(cq->char_on_line == -20)    /* Left one enpty line for new enviroment */
+    line_term = 's';   /* Soft return */
+    if(char_on_line == -20)    /* Left one enpty line for new enviroment */
 	{
-	fputc('%', cq->table);fputc('%', cq->strip);
-	NewLine(cq);
+	fputc('%', table); fputc('%', strip);
+	NewLine(this);
 	}
-    if(cq->char_on_line>=true) //  if(cq->char_on_line>=-1)
+    if(char_on_line>=true) //  if(cq->char_on_line>=-1)
 	{
-	NewLine(cq);
+	NewLine(this);
 	}
-    cq->envir='!';
-    fputc('%', cq->table);fputc('%', cq->strip);
-    NewLine(cq);
+    envir = '!';
+    fputc('%', table); fputc('%', strip);
+    NewLine(this);
 
-    cq->envir = 'B';
+    envir = 'B';
 
-    fprintf(cq->strip, "{|");
-    for (i = 0; i < FieldCount; i++)
+    fprintf(strip, "{|");
+    for(i=0; i<FieldCount; i++)
 	{
 	Alignment='l';
 	if(FieldCount<100 && FieldPos!=NULL) Alignment=FieldPos[i];
-	fprintf(cq->strip, "%c|",Alignment);
+	fprintf(strip, "%c|",Alignment);
 	}
-    putc('}', cq->strip);
+    putc('}', strip);
 
-    cq->char_on_line = false;
-    cq->nomore_valid_tabs = false;
-    cq->rownum++;
-    Make_tableentry_attr(cq);
-    cq->latex_tabpos = 0;
+    char_on_line = false;
+    nomore_valid_tabs = false;
+    rownum++;
+    Make_tableentry_attr(this);
+    latex_tabpos = 0;
     }
 
 	/*Process all content of the table */
-  cq->flag=OldFlag;
-  fseek(cq->wpd,FilePos,SEEK_SET);
-  cq->ReadXMLTag();
-  while(!feof(cq->wpd))
+  flag = OldFlag;
+  fseek(wpd,FilePos,SEEK_SET);
+  ReadXMLTag();
+  while(!feof(wpd))
 	{
-	if((cq->by==0 && (cq->subby==10 || cq->subby==13)))
+	if((by==0 && (subby==10 || subby==13)))
 	  {
-	  cq->by=0;
-	  cq->subby=' ';	/* remove \n from cell text */
+	  by = 0;
+	  subby = ' ';	/* remove \n from cell text */
 	  }
-	ProcessKeyABI(cq);
-	if(cq->by==ABI_TABLE)
-           switch(cq->subby)
+	ProcessKeyABI();
+	if(by==ABI_TABLE)
+           switch(subby)
 		   {
-		   case 0:cq->TablePos=1;goto FinishTable; //nested tables are not converted
-		   case 1:EndTable(cq);
-			  cq->TablePos=0;
+		   case 0:TablePos=1; goto FinishTable; //nested tables are not converted
+		   case 1:EndTable(this);
+			  TablePos=0;
 			  goto FinishTable;
-		   case 2:if((i=cq->TAG_Args IN "props")>=0)
+		   case 2:if((i=TAG_Args IN "props")>=0)
                             {
-                            ParseProps(cq->TAG_Args.Member(i,1),PrpL);
+                            ParseProps(TAG_Args.Member(i,1),PrpL);
                             if((i=PrpL IN "left-attach")>=0)
 	                      {
 	                      Prop=PrpL.Member(i,1);
 			      if(Prop)
 			       if(!strcmp(Prop,"0"))
 				 {
-				 RowTable(cq);cq->TablePos|=2;
+				 RowTable(this); TablePos|=2;
 				 break;
 				 }
 			      else
-				 {CellTable(cq);cq->TablePos|=4;break;}
+				 {CellTable(this);TablePos|=4;break;}
 			      }
 		            }
 			  break;
 		   }
 
-	cq->ReadXMLTag();
+	ReadXMLTag();
 	}
 
 FinishTable:
-  if(cq->char_on_line <= -10)    /* Left one enpty line for ending enviroment */
+  if(char_on_line <= -10)    /* Left one enpty line for ending enviroment */
 	{
-	fputc('%', cq->table);fputc('%', cq->strip);
-	NewLine(cq);
+	fputc('%', table);fputc('%', strip);
+	NewLine(this);
 	}
-  cq->envir='^';		//Ignore enviroments after table
-  fputc('%', cq->table);
-  NewLine(cq);
-  cq->char_on_line = -10;		// stronger false;
+  envir = '^';		//Ignore enviroments after table
+  fputc('%', table);
+  NewLine(this);
+  char_on_line = -10;		// stronger false;
 
-  cq->recursion--;
+  recursion--;
 
-  cq->flag=OldFlag;
-  cq->envir=OldEnvir;
-cq->TAG="TABLE";
-strcpy(cq->ObjType, "Table Start");
+  flag=OldFlag;
+  envir=OldEnvir;
+  TAG="TABLE";
+strcpy(ObjType, "Table Start");
 }
 
 
-static void ProcessKeyABI(TconvertedPass1_XML *cq)
+void TconvertedPass1_Abi::ProcessKeyABI(void)
 {
 #ifdef DEBUG
-  fprintf(cq->log,"\n#ProcessKeyABI() ");fflush(cq->log);
+  fprintf(log,"\n#ProcessKeyABI() ");fflush(log);
 #endif
 string TAG;
-const char *tag;
-BYTE by,subby;
+const char *tag = NULL;
+BYTE by, subby;
 
- *cq->ObjType=0;
- if(cq->TAG=="") cq->ReadXMLTag();
- by=cq->by;
- subby=cq->subby;
+ *ObjType = 0;
+ if(TAG=="") ReadXMLTag();
+ by = this->by;
+ subby = this->subby;
 
  switch(by)
      {
      case XML_char:
-             switch(cq->subby)                        //Normal character
+             switch(this->subby)                        //Normal character
 	       {
 	       case 10:
 	       case 13:by=128;break; //CR
-	       case  9:strcpy(cq->ObjType, "!Tab");
+	       case  9:strcpy(ObjType, "!Tab");
 	       case 32:by=32;break;  //Space
 	       }
              break;
 	case XML_extchar:
-             if(cq->TAG=="") break;			//Extended chatacter &xxx;
-	       TAG = Ext_chr_str(cq->TAG[0],cq)+copy(cq->TAG,1,length(cq->TAG)-1);
+             if(TAG=="") break;			//Extended chatacter &xxx;
+	       TAG = Ext_chr_str(TAG[0],this) + copy(TAG,1,length(TAG)-1);
 	       tag=TAG();
 	       by=201;
 	       break;
 	case XML_tag:
-               TAG = copy(cq->TAG,1,length(cq->TAG)-2);	//Normal tag <xxx>
+               TAG = copy(TAG,1,length(TAG)-2);	//Normal tag <xxx>
 	       if( (tag=TAG())==NULL) break;
 	       if(TAG=="CELL") {by=ABI_TABLE;subby=2;break;}
 	       if(TAG=="D")    {by=ABI_DATA;subby=0;break;}	//data
@@ -617,7 +628,7 @@ BYTE by,subby;
 	       if(TAG=="METADATA"){by=ABI_SECTMETA;subby=0;break;}
 	       break;
 	case XML_closetag:
-               TAG=copy(cq->TAG,2,length(cq->TAG)-3);	//Closing tag </xxx>
+               TAG = copy(TAG,2,length(TAG)-3);	//Closing tag </xxx>
 	       if( (tag=TAG())==NULL) break;
 	       if(TAG=="CELL") {by=ABI_TABLE;subby=3;break;}
 	       if(TAG=="D")    {by=ABI_DATA;subby=1;break;}	//data
@@ -631,91 +642,91 @@ BYTE by,subby;
 	case XML_comment:			//comment
 	       break;
 	case XML_badextchar:
-               if(cq->TAG=="") break;		//Extended chatacter &xxx
-	       cq->TAG[length(cq->TAG)-1]=' ';
-	       TAG=Ext_chr_str(cq->TAG[0],cq)+copy(cq->TAG,1,length(cq->TAG)-1);
-	       tag=TAG();
-	       by=201;
+               if(TAG=="") break;		//Extended chatacter &xxx
+	       TAG[length(TAG)-1] = ' ';
+	       TAG=Ext_chr_str(TAG[0],this) + copy(TAG,1,length(TAG)-1);
+	       tag = TAG();
+	       by = 201;
 	       break;
 	case 6:				//expanded unicode character
 	       break;
 	}
 
-  cq->by=by;
-  cq->subby=subby;
-  if(cq->flag<Nothing)
+  this->by=by;
+  this->subby=subby;
+  if(flag<Nothing)
     switch(by)
 	{
 	case XML_char:
-               tag = Ext_chr_str(subby,cq,cq->ConvertCpg);
-	       CharacterStr(cq,tag);
+               tag = Ext_chr_str(subby,this,ConvertCpg);
+	       CharacterStr(this,tag);
 	       break;		//Normal character
 	case XML_comment:
-               if(cq->rownum<10)
+               if(rownum<10)
 	         {		// Strip leading warning, because it no longer makes sense.
-		 if(strstr(cq->TAG(),"<!-- This file is an AbiWord document.")) break;
-		 if(strstr(cq->TAG(),"<!-- AbiWord is a free, Open Source word processor.")) break;
-		 if(strstr(cq->TAG(),"<!-- More information about AbiWord is available at http://www.abisource.com/")) break;
-		 if(strstr(cq->TAG(),"<!-- You should not edit this file by hand.")) break;
+		 if(StrStr(TAG(),"<!-- This file is an AbiWord document.")) break;
+		 if(StrStr(TAG(),"<!-- AbiWord is a free, Open Source word processor.")) break;
+		 if(StrStr(TAG(),"<!-- More information about AbiWord is available at http://www.abisource.com/")) break;
+		 if(StrStr(TAG(),"<!-- You should not edit this file by hand.")) break;
 		 }     // no break here...
         case XML_CDATA:
-	       cq->CommentXML();
+	       CommentXML();
 	       break;
-	case XML_unicode: CharacterStr(cq,cq->TAG);
+	case XML_unicode: CharacterStr(this,TAG);
 	       break;		//Already expanded unicode character
-	case 32:putc(' ', cq->strip);   /*soft space*/
+	case 32:putc(' ', strip);   /*soft space*/
 		break;
 
-	case 128:if(cq->TablePos!=1 && cq->TablePos!=3)
-		   if(cq->char_on_line)
-			SoftReturn(cq);
+	case 128:if(TablePos!=1 && TablePos!=3)
+		   if(char_on_line)
+			SoftReturn(this);
 		 break;
-	case 129:AttrOn(cq->attr,subby);break;
-	case 130:AttrOff(cq,subby);     break;
-	case ABI_HRT:HardReturn(cq);    break;
-	case ABI_PAR:if(cq->envir=='B') break;
-		 if(subby==0) ParPropAbi(cq);
+	case 129:AttrOn(attr,subby);break;
+	case 130:AttrOff(this,subby);     break;
+	case ABI_HRT:HardReturn(this);    break;
+	case ABI_PAR:if(envir=='B') break;
+		 if(subby==0) ParPropAbi();
 		 if(subby==1)
-		    if(cq->char_on_line) HardReturn(cq);  //Paragraph off
+		    if(char_on_line) HardReturn(this);  //Paragraph off
 		 break;
-	case ABI_FIELD:FieldAbi(cq);		break;
-	case ABI_CHAR:CharPropAbi(cq);		break;
-	case ABI_IMAGE:ImageAbi(cq);		break;
-	case ABI_FRAME:if(!subby) FrameAbi(cq); break;
-	case ABI_SECTION:SectionAbi(cq);	break;
-	case ABI_PBR:HardPage(cq);		break;
-	case ABI_DATA:if(subby==0) ProcessDataAbi(cq);
+	case ABI_FIELD:FieldAbi();		break;
+	case ABI_CHAR:CharPropAbi();		break;
+	case ABI_IMAGE:ImageAbi();		break;
+	case ABI_FRAME:if(!subby) FrameAbi();	break;
+	case ABI_SECTION:SectionAbi(this);	break;
+	case ABI_PBR:HardPage(this);		break;
+	case ABI_DATA:if(subby==0) ProcessDataAbi();
 		 break;
-	case ABI_TABLE:if(subby==0) TableABI(cq);
+	case ABI_TABLE:if(subby==0) TableABI();
 		 break;
-	case ABI_META:if(subby==0) SkipMetaAbi(cq);
+	case ABI_META:if(subby==0) SkipMetaAbi();
     		 break;
 
-	case 200:fputc('~', cq->strip);strcpy(cq->ObjType, " ");
+	case 200:fputc('~', strip);strcpy(ObjType, " ");
 		 break;
-	case 201:CharacterStr(cq,tag);break; //
+	case 201:CharacterStr(this,tag);break; //
 	}
 
 
- cq->by=by;
- cq->subby=subby;
- if (cq->log != NULL)
+ this->by = by;
+ this->subby = subby;
+ if(log != NULL)
     {   /**/
-    if(by==128) fputc('\n',cq->log);
-    else if(by==' ' || by==200) fputc(' ',cq->log);
+    if(by==128) fputc('\n',log);
+    else if(by==' ' || by==200) fputc(' ',log);
     else if(by==0 || by==201)
 	{
-	fprintf(cq->log,"%s",tag);
+	if(tag) fputs(tag,log);
 	}
     else
 	{
-	fprintf(cq->log, _("\n%*s [%s %s]   "),
-		  cq->recursion * 2, "", cq->TAG(), cq->ObjType);
+	fprintf(log, _("\n%*s [%s %s]   "),
+		  recursion * 2, "", TAG(), ObjType);
 //	if(*cq->ObjType==0) UnknownObjects++;
 	}
     }
 
- cq->ActualPos = ftell(cq->wpd);
+ ActualPos = ftell(wpd);
 }
 
 
@@ -747,7 +758,7 @@ DWORD fsize;
 	      perc.Actualise(ActualPos);
 
       TAG = "";
-      ProcessKeyABI(this);
+      ProcessKeyABI();
       }
 
   Finalise_Conversion(this);
