@@ -11,10 +11,10 @@
 #include <string.h>
 #include <stdio.h>
 #include <stdlib.h>
-#include <stdarg.h>
 
 //Atoms library
 #include "stringa.h"
+#include "struct.h"
 
 #include "wp2latex.h"
 #include "images/raster.h"
@@ -202,7 +202,10 @@ void ObjectTable::AddObject(VectorAttribute *NewAttr)
   }
 
   Count++;
-  Attributes = (VectorAttribute**)realloc(Attributes, Count*sizeof(VectorAttribute*));
+  VectorAttribute **tmp = (VectorAttribute**)realloc(Attributes, Count*sizeof(VectorAttribute*));
+  if(tmp==NULL)		// memory exhausted, but Attributes contains original array.
+    {ReportNoMemory();return;}
+  Attributes = tmp;
   Attributes[Count-1] = NewAttr;
 }
 
@@ -268,7 +271,7 @@ public:
      //TconvertedPass1_WMF(void);
 
      virtual int Convert_first_pass(void);
-     virtual int Dispatch(int FuncNo, ...);
+     virtual int Dispatch(int FuncNo, const void *arg);
 
      _StandardMetaRecord WmfRec;
 
@@ -342,6 +345,9 @@ bool LoadWmfRecord(FILE *f, _StandardMetaRecord &WmfRec)
 
 void TconvertedPass1_WMF::ReportCorruptedObj(const char *ObjName)
 {
+#ifdef DEBUG
+  fprintf(log,"\n#TconvertedPass1_WMF::ReportCorruptedObj() ");fflush(log);
+#endif
   if(err==NULL) return;
   perc.Hide();
   fprintf(err,_("\nError: corrupted wmf object: \"%s\"."), ObjName);
@@ -350,6 +356,9 @@ void TconvertedPass1_WMF::ReportCorruptedObj(const char *ObjName)
 
 void TconvertedPass1_WMF::wmf_ESCAPE(void)
 {
+#ifdef DEBUG
+  fprintf(log,"\n#TconvertedPass1_WMF::wmf_ESCAPE() ");fflush(log);
+#endif
 static WORD MathType[] = {15,12,'M'+256*'a','t'+256*'h','T'+256*'y','p'+256*'e',0};
 int Size;
 WORD w;
@@ -380,29 +389,28 @@ NoMTEF:
 
 
 /** This callback from parent allows to provide some generic functionality. */
-int TconvertedPass1_WMF::Dispatch(int FuncNo, ...)
+int TconvertedPass1_WMF::Dispatch(int FuncNo, const void *arg)
 {
 #ifdef DEBUG
   fprintf(log,"\n#TconvertedPass1_WMF::Dispatch(%d) ",FuncNo);fflush(log);
 #endif
-va_list ap;
 
- va_start(ap, FuncNo); 
  switch(FuncNo)
    {
    case DISP_EXTRACTIMAGE:
          {
-	 Image **FillME = va_arg(ap,Image **);
+	 Image **FillME = (Image**)arg;
          if(FillME) *FillME = new Image(Img);
-         va_end(ap);
 	 return 0;
 	 }
    case DISP_NOCONVERTIMAGE:
-	 NoConvertImage = va_arg(ap,int);
-         va_end(ap);
-         return 0;
-   } 
-  va_end(ap);
+         if(arg!=NULL)
+         {
+	   NoConvertImage = *(int*)arg;         
+           return 0;
+         }
+         break;
+   }
 
 return(-1);
 }
@@ -412,10 +420,13 @@ return(-1);
  * @param[in]	WSizeRequest	Capacity of data payload in WORDs, without header record. */
 bool TconvertedPass1_WMF::CheckWmfRecSz(WORD WSizeRequest)
 {
+#ifdef DEBUG
+  fprintf(log,"\n#TconvertedPass1_WMF::CheckWmfRecSz() ");fflush(log);
+#endif
   if(WSizeRequest > WmfRec.Size-3)	// {RecordSize}[RecordFunction]
   {
     if(err != NULL)
-        fprintf(err,_("\nError: WMF record %X size %d is too small, expected %d!"), 
+        fprintf(err,_("\nError: WMF record %X size %d is too small, expected %u!"), 
                 WmfRec.ParamFilePos ,WmfRec.Size ,WSizeRequest+2);
     return false;
   }
@@ -425,6 +436,9 @@ bool TconvertedPass1_WMF::CheckWmfRecSz(WORD WSizeRequest)
 
 void TconvertedPass1_WMF::parse_Ellipse(VectorList &VectList)
 {
+#ifdef DEBUG
+  fprintf(log,"\n#TconvertedPass1_WMF::parse_Ellipse() ");fflush(log);
+#endif
 static const char ObjName[] = "!Ellipse";
 SWORD BottomRect, TopRect, RightRect, LeftRect;
 
@@ -455,6 +469,9 @@ strcpy(ObjType,ObjName+1);
 
 void TconvertedPass1_WMF::parse_Chord(void)
 {
+#ifdef DEBUG
+  fprintf(log,"\n#TconvertedPass1_WMF::parse_Chord() ");fflush(log);
+#endif
 static const char ObjName[] = "!Chord";
 SWORD	XRadial1, YRadial1, XRadial2, YRadial2;
 SWORD	BottomRect, RightRect, TopRect, LeftRect;
@@ -481,9 +498,12 @@ strcpy(ObjType,ObjName+1);    // @TODO: dopsat!
 
 void TconvertedPass1_WMF::parse_FontIndirect(void)
 {
+#ifdef DEBUG
+  fprintf(log,"\n#TconvertedPass1_WMF::parse_FontIndirect() ");fflush(log);
+#endif
 static const char ObjName[] = "!CreateFontIndirect";
 SWORD Height;
-WORD Width, Escapement, Orientation;
+WORD Width, Escapement;
 BYTE Underline, Strikeout, Charset, OutPrecision, ClipPrecision, Quality, PitchAndFamily;
 
   if(!CheckWmfRecSz(9))
@@ -496,7 +516,7 @@ BYTE Underline, Strikeout, Charset, OutPrecision, ClipPrecision, Quality, PitchA
   Rd_word(wpd, (WORD*)&Height);
   Rd_word(wpd, &Width);
   Rd_word(wpd, &Escapement);
-  Rd_word(wpd, &Orientation);
+  Rd_word(wpd, (WORD*)&PSS.FontOrientation10);
   Rd_word(wpd, &PSS.FontWeight);
   PSS.FontItallic = fgetc(wpd);
   Underline = fgetc(wpd);
@@ -553,13 +573,16 @@ sprintf(ObjType,"%s(%d)", ObjName+1, (int)Height);
 
 void TconvertedPass1_WMF::parse_MoveTo(void)
 {
+#ifdef DEBUG
+  fprintf(log,"\n#TconvertedPass1_WMF::parse_MoveTo() ");fflush(log);
+#endif
 static const char ObjName[] = "!MoveTo";
 
   if(CheckWmfRecSz(2))
     {
     Rd_word(wpd, (WORD*)&PositionY);
     Rd_word(wpd, (WORD*)&PositionX);
-    sprintf(ObjType,"%s(%u;%u)", ObjName+1, PositionX, PositionY);
+    sprintf(ObjType,"%s(%d;%d)", ObjName+1, PositionX, PositionY);
     }
   else
     strcpy(ObjType,ObjName);  
@@ -568,6 +591,9 @@ static const char ObjName[] = "!MoveTo";
 
 void TconvertedPass1_WMF::parse_CreateBrushIndirect(void)
 {
+#ifdef DEBUG
+  fprintf(log,"\n#TconvertedPass1_WMF::parse_CreateBrushIndirect() ");fflush(log);
+#endif
 static const char ObjName[] = "!CreateBrushIndirect";
 WORD BrushStyle;
   if(!CheckWmfRecSz(3))
@@ -597,6 +623,9 @@ WORD BrushStyle;
 
 void TconvertedPass1_WMF::parse_CreatePenIndirect(void)
 {
+#ifdef DEBUG
+  fprintf(log,"\n#TconvertedPass1_WMF::parse_CreatePenIndirect() ");fflush(log);
+#endif
 static const char ObjName[] = "!CreatePenIndirect";
 WORD PenStyle, PenWidthX, PenWidthY;
   if(!CheckWmfRecSz(5))
@@ -637,6 +666,9 @@ WORD PenStyle, PenWidthX, PenWidthY;
 
 void TconvertedPass1_WMF::parse_Rectangle(VectorList &VectList)
 {
+#ifdef DEBUG
+  fprintf(log,"\n#TconvertedPass1_WMF::parse_Rectangle() ");fflush(log);
+#endif
 static const char ObjName[] = "!Rectangle";
 SWORD	BottomRect, TopRect, RightRect, LeftRect;
 
@@ -656,7 +688,7 @@ SWORD	BottomRect, TopRect, RightRect, LeftRect;
 
   const float Scale = GetScale2PSU((TMapMode)MapMode);
 
-  VectorRectangle *pVectRec = new VectorRectangle(Scale*BottomRect, Scale*TopRect, Scale*RightRect, Scale*LeftRect);
+  VectorRectangle *pVectRec = new VectorRectangle(Scale*BottomRect, Scale*TopRect, Scale*LeftRect, Scale*RightRect);
   pVectRec->AttribFromPSS(PSS);    
   VectList.AddObject(pVectRec);  
 
@@ -666,6 +698,9 @@ SWORD	BottomRect, TopRect, RightRect, LeftRect;
 
 void TconvertedPass1_WMF::parse_DeleteObject(void)
 {
+#ifdef DEBUG
+  fprintf(log,"\n#TconvertedPass1_WMF::parse_DeleteObject() ");fflush(log);
+#endif
 WORD ObjectIndex;
 static const char ObjName[] = "!DeleteObject";
 
@@ -684,6 +719,9 @@ static const char ObjName[] = "!DeleteObject";
 
 void TconvertedPass1_WMF::parse_SelectObject(void)
 {
+#ifdef DEBUG
+  fprintf(log,"\n#TconvertedPass1_WMF::parse_SelectObject() ");fflush(log);
+#endif
 WORD ObjectIndex;
 static const char ObjName[] = "!SelectObject";
 
@@ -711,6 +749,9 @@ static const char ObjName[] = "!SelectObject";
 
 void TconvertedPass1_WMF::parse_SetBkColor(void)
 {
+#ifdef DEBUG
+  fprintf(log,"\n#TconvertedPass1_WMF::parse_SetBkColor() ");fflush(log);
+#endif
   if(CheckWmfRecSz(2))
     {
     PSS.FillBackground.Red   = fgetc(wpd);
@@ -724,6 +765,9 @@ void TconvertedPass1_WMF::parse_SetBkColor(void)
 
 void TconvertedPass1_WMF::parse_SetTextColor(void)
 {
+#ifdef DEBUG
+  fprintf(log,"\n#TconvertedPass1_WMF::parse_SetTextColor() ");fflush(log);
+#endif
   if(CheckWmfRecSz(2))
     {
     PSS.TextColor.Red   = fgetc(wpd);
@@ -737,6 +781,9 @@ void TconvertedPass1_WMF::parse_SetTextColor(void)
 
 void TconvertedPass1_WMF::parse_SetWindowExt(void)
 {
+#ifdef DEBUG
+  fprintf(log,"\n#TconvertedPass1_WMF::parse_SetWindowExt() ");fflush(log);
+#endif
   Rd_word(wpd, (WORD*)&YExtent);
   Rd_word(wpd, (WORD*)&XExtent);
   sprintf(ObjType,"SetWindowExt(%d,%d)", (int)XExtent, (int)YExtent);
@@ -745,6 +792,9 @@ void TconvertedPass1_WMF::parse_SetWindowExt(void)
 
 void TconvertedPass1_WMF::parse_SetWindowOrg(void)
 {
+#ifdef DEBUG
+  fprintf(log,"\n#TconvertedPass1_WMF::parse_SetWindowOrg() ");fflush(log);
+#endif
   Rd_word(wpd, (WORD*)&YOffset);
   Rd_word(wpd, (WORD*)&XOffset);
   sprintf(ObjType,"SetWindowOrg(%d,%d)", (int)XOffset, (int)YOffset);
@@ -753,6 +803,9 @@ void TconvertedPass1_WMF::parse_SetWindowOrg(void)
 
 void TconvertedPass1_WMF::parse_LineTo(VectorList &VectList)
 {
+#ifdef DEBUG
+  fprintf(log,"\n#TconvertedPass1_WMF::parse_LineTo() ");fflush(log);
+#endif
 static const char ObjName[] = "!LineTo";
 
   if(CheckWmfRecSz(2))
@@ -790,6 +843,9 @@ static const char ObjName[] = "!LineTo";
 
 void TconvertedPass1_WMF::parse_Polygon(VectorList &VectList)
 {
+#ifdef DEBUG
+  fprintf(log,"\n#TconvertedPass1_WMF::parse_Polygon() ");fflush(log);
+#endif
 static const char ObjName[] = "!Polygon";
   WORD PolySize;  
 
@@ -823,8 +879,12 @@ POLY_FAIL:
   return;
 }
 
+
 void TconvertedPass1_WMF::parse_PolyPolygon(VectorList &VectList)
 {
+#ifdef DEBUG
+  fprintf(log,"\n#TconvertedPass1_WMF::parse_PolyPolygon() ");fflush(log);
+#endif
 static const char ObjName[] = "!PolyPolygon";
 WORD PolyCount;
 WORD *PolySizes;
@@ -878,6 +938,9 @@ POLY_FAIL:
 
 void TconvertedPass1_WMF::parse_PolyLine(VectorList &VectList)
 {
+#ifdef DEBUG
+  fprintf(log,"\n#TconvertedPass1_WMF::parse_PolyLine() ");fflush(log);
+#endif
 static const char ObjName[] = "!Polyline";
   WORD PolySize;  
 
@@ -913,6 +976,9 @@ POLY_FAIL:
 
 void TconvertedPass1_WMF::parse_SetMapMode(void)
 {
+#ifdef DEBUG
+  fprintf(log,"\n#TconvertedPass1_WMF::parse_SetMapMode() ");fflush(log);
+#endif
 static const char ObjName[] = "!SetMapMode(%u)";
   if(!CheckWmfRecSz(1))
   {
@@ -927,6 +993,9 @@ static const char ObjName[] = "!SetMapMode(%u)";
 
 void TconvertedPass1_WMF::parse_TextOut(VectorList &VectList)
 {
+#ifdef DEBUG
+  fprintf(log,"\n#TconvertedPass1_WMF::parse_TextOut() ");fflush(log);
+#endif
 static const char ObjName[] = "!TextOut";
 WORD StringLen;
 short int LineLen, CurLineLen;
@@ -948,6 +1017,7 @@ TEXT_OUT_FAIL:
   memcpy(&pTextCont->TextColor, &PSS.TextColor, sizeof(pTextCont->TextColor));
   pTextCont->PosX = 0;
   pTextCont->PosY = 0;
+  pTextCont->FontOrientation = PSS.FontOrientation10 / 10.0f;
   LineLen = CurLineLen = 0;
   XStart = StringLen & 1;
   while(StringLen-->0)
@@ -970,6 +1040,7 @@ TEXT_OUT_FAIL:
         memcpy(&pTextCont->TextColor, &PSS.TextColor, sizeof(pTextCont->TextColor));
         pTextCont->PosX = 0;
         pTextCont->PosY = CurPosY;
+        pTextCont->FontOrientation = PSS.FontOrientation10 / 10.0f;
       }
     }
     else CurLineLen++;
@@ -1022,6 +1093,9 @@ TEXT_OUT_FAIL:
 
 void TconvertedPass1_WMF::parse_ExtTextOut(VectorList &VectList)
 {
+#ifdef DEBUG
+  fprintf(log,"\n#TconvertedPass1_WMF::parse_ExtTextOut() ");fflush(log);
+#endif
 static const char ObjName[] = "!ExtTextOut";
 SWORD x_pos, y_pos;
 WORD fuOptions, TextLen;
@@ -1062,6 +1136,7 @@ unsigned char c;
   memcpy(&pTextCont->TextColor, &PSS.TextColor, sizeof(pTextCont->TextColor));
   pTextCont->PosX = Scale*x_pos;
   pTextCont->PosY = Scale*y_pos;
+  pTextCont->FontOrientation = PSS.FontOrientation10 / 10.0f;
 
   LineLen = CurLineLen = 0;
   if(log!=NULL) fputc(' ',log);
@@ -1083,6 +1158,7 @@ unsigned char c;
         memcpy(&pTextCont->TextColor, &PSS.TextColor, sizeof(pTextCont->TextColor));
         pTextCont->PosX = Scale*x_pos;        
         pTextCont->PosY = CurPosY;
+        pTextCont->FontOrientation = PSS.FontOrientation10 / 10.0f;
       }
     }
     else CurLineLen++;
@@ -1131,6 +1207,9 @@ unsigned char c;
 
 void TconvertedPass1_WMF::StretchDIBits(void)
 {
+#ifdef DEBUG
+  fprintf(log,"\n#TconvertedPass1_WMF::StretchDIBits() ");fflush(log);
+#endif
 Image *pImg = &Img;
 
 	// Rewind to the last frame.
@@ -1199,7 +1278,7 @@ DWORD NewPos;
   if(WmfHead.HeaderSize!=9)
     {
     if(err != NULL)
-        fprintf(err, _("\nError: Invalid WMF header size %d, expected size=9!"), WmfHead.HeaderSize);
+        fprintf(err, _("\nError: Invalid WMF header size %u, expected size=9!"), WmfHead.HeaderSize);
     return(-1);
     }
 
@@ -1349,9 +1428,9 @@ DWORD NewPos;
       if(ObjType[0]=='!') UnknownObjects++;
 
       if(log!=NULL)
-	{   /**/
+	{
 	fprintf(log,_("\n%*s{WMF function:%3Xh; size:%4lu; pos:%lXh}"),
-		  recursion*2,"",WmfRec.Function,2*WmfRec.Size,ActualPos);
+		  recursion*2,"",WmfRec.Function, (long)2*WmfRec.Size, (long)ActualPos);
 	if(*ObjType!=0) fprintf(log," %s",ObjType);
 
         if(ObjType[0] == '!')
@@ -1425,13 +1504,16 @@ DWORD NewPos;
   if(YExtent>=0)
       VectList.Transform(flipTrx);
 
-  PSData += VectList.Export2EPS(&PSS);
-  if(PSData.length()>0 && Img.Comment==NULL)
-  {
-    Img.Comment = PSData.ExtractString();
+  if(PSData.length() > 0)
+      VectList.AddObject(new PsBlob(PSData.ExtractString()));  
+
+  if(VectList.VectorObjects>0 && Img.VecImage==NULL)
+  {    
+    Img.AttachVecImg(new VectorImage(VectList,PSS));
+
     if(Img.dx!=0 && Img.dy!=0 && Img.Raster!=NULL)
     {
-      if(Img.Comment!=NULL)	// Move raster data to different image frame.
+      if(Img.VecImage!=NULL)	// Move raster data to different image frame.
       {
         Img2 = &Img;
         while(Img2->Next!=NULL)
@@ -1443,7 +1525,7 @@ DWORD NewPos;
         Img2->y =  bbx.MinY * Scale;
         Img2->dx = (bbx.MaxX - bbx.MinX) * Scale;
         Img2->dy = (bbx.MaxY - bbx.MinY) * Scale;
-        Img2->Comment = Img.Comment; Img.Comment=NULL;
+        Img2->VecImage = Img.VecImage; Img.VecImage=NULL;
       }
     }
     else	// Use whole frame as bounding box.
@@ -1455,7 +1537,7 @@ DWORD NewPos;
     }        
   }
 
-  if(!NoConvertImage && (Img.Raster!=NULL || Img.Comment!=NULL))
+  if(!NoConvertImage && (Img.Raster!=NULL || Img.VecImage!=NULL))
     {
     for(Image *pImg=&Img; pImg!=NULL; pImg=pImg->Next)
       if(pImg->Raster!=NULL)
@@ -1467,7 +1549,7 @@ DWORD NewPos;
         if(err != NULL)
 	  {
 	  perc.Hide();
-	  fprintf(err, _("\nError: Cannot save file: \"%s\"!"),NewFilename());
+	  fprintf(err, _("\nError: Cannot save file: \"%s\"!"), NewFilename());
 	  }
 	return 0;
         }
@@ -1587,7 +1669,7 @@ int TconvertedPass1_WMFspec::Convert_first_pass(void)
     }
   }
 
- return this->TconvertedPass1_WMF::Convert_first_pass();
+ return TconvertedPass1_WMF::Convert_first_pass();
 }
 
 /* Register translator for WMF with special header here. */
@@ -1929,14 +2011,14 @@ EMFHEAD EmfHead;
       case EMR_SETTEXTJUSTIFICATION: strcpy(ObjType,"!SETTEXTJUSTIFICATION"); break;	// 0x00000078
       case EMR_COLORMATCHTOTARGETW: strcpy(ObjType,"!COLORMATCHTOTARGETW"); break;	// 0x00000079
       case EMR_CREATECOLORSPACEW: strcpy(ObjType,"!CREATECOLORSPACEW"); break;	// 0x0000007A
-      default: sprintf(ObjType,"!Unknown#%X", EmfRec.Type);
+      default: sprintf(ObjType,"!Unknown#%X", (unsigned)EmfRec.Type);
     }
     if(ObjType[0]=='!') UnknownObjects++;
     
     if(log!=NULL)
     {   /**/
       fprintf(log, _("\n%*s{EMF function:%3Xh; size:%4lu; pos:%lXh}"),
-		recursion*2,"",EmfRec.Type, 2*EmfRec.Size, EmfRec.ParamFilePos);
+		recursion*2,"",EmfRec.Type, (long)2*EmfRec.Size, (long)EmfRec.ParamFilePos);
       if(*ObjType!=0) fprintf(log," %s",ObjType);
     }
 
