@@ -16,9 +16,10 @@
 #include <math.h>
 #include <ctype.h>
 
-#include <stringa.h>
-#include <lists.h>
+#include "stringa.h"
+#include "lists.h"
 #include "wp2latex.h"
+#include "struct.h"
 
 extern list Counters, UserLists;
 
@@ -33,6 +34,8 @@ extern list Counters, UserLists;
 #endif
 
 
+#define TABPOS_COUNT	40
+
 /// Attribute for each line of conferted text.
 typedef struct TLinePass2
   {
@@ -50,11 +53,10 @@ typedef struct TLinePass2
 
   int ind_leftmargin;
   signed char Indent_Level;
+  WORD tabent_tabpos[TABPOS_COUNT];
   } TLinePass2;
 
 
-#define TABPOS_COUNT	40
-
 /// Main object that packs all needed stuff for pass2.
 class TconvertedPass2
 {
@@ -66,15 +68,13 @@ public:
   BYTE LangCount;
 
   TLinePass2 Lines[4];
+  WORD tabpos[TABPOS_COUNT];  
 
   bool underline, illegal_argument, just_envir_closed;
 
   int num_of_tabs;
-  char Col2Break;			///< Amount of \columnbreak inside multicol environment
-  unsigned short tabpos[TABPOS_COUNT];
-
-  unsigned int tabent_tabpos[4][TABPOS_COUNT];
-
+  char Col2Break;			///< Amount of \columnbreak inside multicol environment  
+  
   short int pre, cur, next, next2;
   FILE *table;
   FILE *Strip, *LaTeX;
@@ -189,7 +189,7 @@ static bool SectionOrEmpty(const char *str)
 void TconvertedPass2::ReadTableEntry(short n)
 {
 #ifdef DEBUG
-  fprintf(log,"\n#Read_TableEntry(%d) ",(int)n);fflush(log);
+  fprintf(log,"\n#Read_TableEntry(%d) ",(int)n); fflush(log);
 #endif
 WORD w;
 BYTE b;
@@ -201,7 +201,7 @@ TLinePass2 *pCurLine;
   pCurLine->LineFlag = 0;
   pCurLine->new_tabset = false;
 
-  InitAttr(pCurLine->attr);
+  pCurLine->attr.InitAttr();
 
   if(feof(table)) return;
 
@@ -221,9 +221,9 @@ TLinePass2 *pCurLine;
 	case 'q':pCurLine->envir = 'q';    break;
 	case 'Q':pCurLine->envir = 'Q';    break;
 	case 'I':
-	case 'i':pCurLine->envir=b;
+	case 'i':pCurLine->envir = b;
 		 Rd_word(table, &w);
-		 pCurLine->ind_leftmargin=w;
+		 pCurLine->ind_leftmargin = w;
 		 fread(&pCurLine->Indent_Level, 1, 1, table);
 		 break;
 
@@ -233,14 +233,14 @@ TLinePass2 *pCurLine;
 		 for(j=0; j<b; j++)
 		     {
 		     Rd_word(table, &w);
-		     tabent_tabpos[n][j] = w;
+		     pCurLine->tabent_tabpos[j] = w;
 		     }
 		 break;
 
        case 'l': Rd_word(table, &w);
 		 if(w!=LineSpacing)
 			    {
-			    fprintf(LaTeX,"\\baselineskip=%2.2fex ",float(w)/128);
+			    fprintf(LaTeX,"\\baselineskip=%2.2fex ",(float)(w)/128.0);
 			    LineSpacing=w;
 			    }
 		  break;
@@ -261,7 +261,7 @@ TLinePass2 *pCurLine;
 		    // Read all opened attributes into attr structure
 	case 0xF0:fread(&b, 1, 1, table);
 		  pCurLine->attr.Opened_Depth=pCurLine->attr.Closed_Depth = b;
-		  for (j = 0; j < b; j++)
+		  for(j = 0; j<b; j++)
 			fread(&(pCurLine->attr.stack[j]), 1, 1, table);
 		  b=0xF0;
 		  break;
@@ -292,7 +292,7 @@ void TconvertedPass2::LatexHead(void)
 int i;
 string TeX_RelativeFigDir;
 
-  fprintf(this->LaTeX,_("%% This file was created by the WP2LaTeX program version: %s \n"), version );
+  fprintf(this->LaTeX,_("%% This file was created by the WP2LaTeX program version: %s \n"), VersionWP2L );
   if(LaTeX_Version<0x300)
     {
     fprintf(this->LaTeX, "\\documentstyle[11pt,");
@@ -302,6 +302,7 @@ string TeX_RelativeFigDir;
     if(MakeIdx>=1)  fprintf(this->LaTeX, "makeidx,");
     if(Ulem>=1)     fprintf(this->LaTeX, "ulem,");
     if(Rotate>=1)   fprintf(this->LaTeX, "rotate,");
+    if(Scalerel>=1) fprintf(this->LaTeX, "scalerel,");
     if(LineNo>=1)   fprintf(this->LaTeX, "lineno,");
     fprintf(this->LaTeX, "wp2latex");
     if(twoside)     fprintf(this->LaTeX, ",twoside");
@@ -344,6 +345,7 @@ string TeX_RelativeFigDir;
     if(MakeIdx>=1)  UsePackage("makeidx");
     if(Ulem>=1)     UsePackage("ulem");
     if(Rotate>=1)   UsePackage("rotate");
+    if(Scalerel>=1) UsePackage("scalerel");
     if(LineNo>=1)   UsePackage("lineno");
     UsePackage("wp2latex");
     if(CPStyles) switch(CodePage)
@@ -471,9 +473,9 @@ void TconvertedPass2::Latex_tabset(void)
 
 	// Huiding number of tabs per row
   FORLIM = num_of_tabs;
-  for (j = 0; j < FORLIM; j++)
+  for(j=0; j<FORLIM; j++)
      {
-     l = tabpos[j] / 1200.0 * 2.54;
+     l = (double)(tabpos[j]) / 1200.0 * 2.54;
      fprintf(LaTeX, "\\hspace{%3.2fcm}\\=", l - ol);
      atpr++;
      if (atpr >= 4)
@@ -615,7 +617,7 @@ bool TconvertedPass2::OpenEnvironment(void)
 	     case 1:fprintf(LaTeX, "\\onecolumn\n"); break;
 	     case 2:fprintf(LaTeX, "\\twocolumn\n"); break;
 	     default:if(err!=NULL)
-			fprintf(err,_("\nError: Bad number of columns:%d !"),(int)Lines[cur].Columns);
+			fprintf(err,_("\nError: Bad number of columns: %d!"),(int)Lines[cur].Columns);
 	     }
 	  }
 	}
@@ -688,16 +690,15 @@ void TconvertedPass2::Update_global_information(void)
 #ifdef DEBUG
   fprintf(log,"\n#Update_global_information() ");fflush(log);
 #endif
-  int j, FORLIM;
-
-  if(Lines[cur].new_tabset)
-        {
-	num_of_tabs = Lines[cur].tabent_num_of_tabs;
-        FORLIM = num_of_tabs;
-        for (j = 0; j < FORLIM; j++)
-        tabpos[j] = tabent_tabpos[cur][j];
-	}
+int j;
+const TLinePass2 *pCurLine = &Lines[cur];
 
+  if(pCurLine->new_tabset)
+    {
+    num_of_tabs = pCurLine->tabent_num_of_tabs;    
+    for(j=0; j<num_of_tabs; j++)
+          tabpos[j] = pCurLine->tabent_tabpos[j];
+    }
 }
 
 
@@ -752,13 +753,13 @@ bool TconvertedPass2::FixTabs(char *Helps)
   if(Lines[cur].envir == 'T')
      {
      j=0;
-     for(i=0;i<tabs;i++)
+     for(i=0; i<tabs; i++)
   	   {
 	   Helps[j++]='\\';
 	   Helps[j++]='>';
            }
-     i=5*tabs;
-     while (Helps[i] != '\0')
+     i = 5*tabs;
+     while(Helps[i] != '\0')
             {
 	    Helps[j] = Helps[i];
 	    i++;
@@ -769,7 +770,7 @@ bool TconvertedPass2::FixTabs(char *Helps)
   else {
        i = Helps - Lines[cur].Lstring();
        s = copy(Lines[cur].Lstring,0,i);
-       s.cat_printf("\\hspace*{%2.2fcm}", float(tabpos[tabs-1])/ 470.0);
+       s.cat_printf("\\hspace*{%2.2fcm}", (float)(tabpos[tabs-1])/ 470.0);
        s += copy(Lines[cur].Lstring,i + 5*tabs,length(Lines[cur].Lstring)- i - 5*tabs);
        Lines[cur].Lstring=s;
        }
@@ -1344,14 +1345,13 @@ while(*aLinePtr!=0)
 	     aLinePtr+=strlen("\\penalty-10001");
 	     while(aLinePtr[i]==' ')
 		 {
+		 i++;
 		 if(aLinePtr[i]==0)   goto Reject;
 		 if(aLinePtr[i]=='&') goto Reject;
 		 if(aLinePtr[i]=='\\')
 		   {
 		   if(aLinePtr[++i]=='\\') goto Reject;
-		   }
-
-		 i++;
+		   }		 
 		 }
 
 	     if(!shortstack) Cell="\\shortstack{"+Cell;
@@ -1520,10 +1520,9 @@ TconvertedPass2 cq2;
       cq2.Lines[i].Empty   = true;
       cq2.Lines[i].Columns = 1;
       cq2.Lines[i].InhibitBsBslash = false;
-      InitAttr(cq2.Lines[i].attr);
+      cq2.Lines[i].attr.InitAttr();
       }
 
-
   cq2.rownum = 1;
   cq2.perc.Init(0, num_of_lines_stripfile, _("Second pass:"));
 
@@ -1727,7 +1726,7 @@ TconvertedPass2 cq2;
 
 
 		//fix closed attributes problem
-    if(ClosedAttr(cq2.Lines[cq2.cur].attr)) /* Are attributes closed? */
+    if(cq2.Lines[cq2.cur].attr.ClosedAttr()) /* Are attributes closed? */
       if(*aLineStr == "")
 	 {
 	 if(cq2.Lines[cq2.next].attr.Closed_Depth==cq2.Lines[cq2.cur].attr.Closed_Depth)
@@ -1853,7 +1852,7 @@ FixOpen:	  Open_All_Attr(cq2.Lines[cq2.cur].attr,*aLineStr);
 				       cq2.Lines[cq2.next].attr.stack[i] == 0xe);  //Underline
 
 		    if (cq2.underline && (cq2.Lines[cq2.next].attr.Opened_Depth>0) &&
-			  !ClosedAttr(cq2.Lines[cq2.cur].attr) )
+			  !cq2.Lines[cq2.cur].attr.ClosedAttr() )
 		      {
 		      Close_All_Attr(cq2.Lines[cq2.next].attr,cq2.LaTeX); /* close attributes on EOL */
 		      }
