@@ -20,11 +20,12 @@
 #endif
 
 //Non standard include files
-#include <common.h>
-#include <stringa.h>
-#include <lists.h>
+#include "common.h"
+#include "stringa.h"
+#include "lists.h"
 
 #include "wp2latex.h"
+#include "struct.h"
 #include "images/raster.h"
 
 #ifdef __BORLANDC__
@@ -101,13 +102,15 @@ SBYTE   Accents = false,
 	LongTable = -1,
 	SaveWPG = -1,
         Rotate = false,
+        Scalerel = false,
 	LineNo = false;
 bool Index = false,
+        Interactive = true,
 	NFSS = false,
 	TABs = true,
 	twoside = false,
 	ExactFnNumbers = false,
-	CPStyles = false;
+	CPStyles = false;        
 
 string force, FORCE;
 int LaTeX_Version=0x300; /* 0x209=LaTeX 2.09; 0x300=LaTex 3.0 (or 2e) */
@@ -120,6 +123,9 @@ int UseMemStream=0;
 
 bool OptimizeSection;
 signed char Verbosing;	  ///< <=-1 no errors, <=0 no warnings, 1 normal, >1 verbosive
+#ifdef _REENTRANT
+short JobCount = -1;
+#endif
 bool TexChars;
 bool ExtendedCheck, FixSpaces;
 bool EraseStrip=true;
@@ -411,8 +417,9 @@ static int DetectAccent(FILE *text, FFormatDetect & iFFD)
 int Status,CharNo=0,chr;
 string s;
 
-  iFFD.DocumentType="Accent";
-  iFFD.Convertor=iFFD.DocumentType;
+  iFFD.DocumentType = "Accent";
+  iFFD.Convertor = iFFD.DocumentType;
+  iFFD.Extension = "acc";
   if(text==NULL) return(0);
 
   Status=0;
@@ -461,7 +468,7 @@ static int DetectBMP(FILE *stream, FFormatDetect & iFFD)
 char buffer[2];
 
   iFFD.DocumentType="BMP image";
-  iFFD.Convertor="BMP";
+  iFFD.Extension = iFFD.Convertor = "BMP";
   iFFD.Probability=-1;
   iFFD.DocVersion=0;
 
@@ -483,7 +490,8 @@ static int DetectHtml(FILE *text, FFormatDetect & iFFD)
 int Status,CharNo=0,chr;
 
   iFFD.DocumentType="HTML";
-  iFFD.Convertor=iFFD.DocumentType;
+  iFFD.Convertor = iFFD.DocumentType;
+  iFFD.Extension = "htm";
   if(text==NULL) return(0);
 
   Status=0;
@@ -548,8 +556,9 @@ static int DetectMTEF(FILE *stream, FFormatDetect & iFFD)
 DWORD Version,ObjLen,FileSize;
 WORD HdrLen;
 
-  iFFD.DocumentType="MTEF";
-  iFFD.Convertor=iFFD.DocumentType;
+  iFFD.DocumentType = "MTEF";
+  iFFD.Convertor = iFFD.DocumentType;
+  iFFD.Extension = "MTF";
   if(stream==NULL) return(0);
 
   FileSize=filesize(stream);
@@ -583,7 +592,7 @@ static int DetectPostScript(FILE *stream, FFormatDetect & iFFD)
 WORD Magic1,Magic2;
 
   iFFD.DocumentType="PostScript file";
-  iFFD.Convertor="PS";
+  iFFD.Extension = iFFD.Convertor = "PS";
   if(stream==NULL) return 0;
 
   Rd_word(stream, &Magic1);
@@ -604,8 +613,8 @@ static int DetectRTF(FILE *text, FFormatDetect & iFFD)
 long OldFilePos;
 char buffer[6];
 
-  iFFD.DocumentType="RTF";
-  iFFD.Convertor=iFFD.DocumentType;
+  iFFD.DocumentType = "RTF";
+  iFFD.Extension = iFFD.Convertor = iFFD.DocumentType;
 
   if(text==NULL) return(0);
 
@@ -631,8 +640,8 @@ long OldFilePos;
 string Command;
 int commands=0,len;
 
-  iFFD.DocumentType="Text 602";
-  iFFD.Convertor="T602";
+  iFFD.DocumentType = "Text 602";
+  iFFD.Extension = iFFD.Convertor = "T602";
   if(text==NULL) return(0);
   OldFilePos=ftell(text);
 
@@ -667,8 +676,8 @@ static int DetectWinMetafile(FILE *stream, FFormatDetect & iFFD)
 {
 DWORD Magic1,Magic2;
 
-  iFFD.DocumentType="Windows Metafile";
-  iFFD.Convertor="WMF";
+  iFFD.DocumentType = "Windows Metafile";
+  iFFD.Extension = iFFD.Convertor = "WMF";
   if(stream==NULL) return 0;
 
   Rd_dword(stream, &Magic1);
@@ -683,7 +692,7 @@ DWORD Magic1,Magic2;
   if(Magic1==0x00090001) return 1;	//This is a WMF
 
   iFFD.DocumentType="Enhanced Metafile";
-  iFFD.Convertor="EMF";
+  iFFD.Extension = iFFD.Convertor = "EMF";
 
   if(Magic1==0x464D4520 && Magic2==0x00000100) return 1; //This is EMF
 
@@ -706,8 +715,8 @@ int Status,CharNo=0,chr;
 static const char *XML="?xml version";
 static const char *DocType="!DOCTYPE";
 
-  iFFD.DocumentType="XML";
-  iFFD.Convertor=iFFD.DocumentType;
+  iFFD.DocumentType = "XML";
+  iFFD.Extension = iFFD.Convertor = iFFD.DocumentType;
   if(text==NULL) return(0);
 
   Status=0;
@@ -750,6 +759,7 @@ static const char *DocType="!DOCTYPE";
 			   fseek(text,0,SEEK_SET);
 			   iFFD.DocumentType="abiword";
 			   iFFD.Convertor=iFFD.DocumentType;
+                           iFFD.Extension = "abw";
 			   return(2);
 			   }
 		       break;
@@ -763,8 +773,9 @@ static const char *DocType="!DOCTYPE";
 		case 23:if(chr=='>')
 			   {
 			   fseek(text,0,SEEK_SET);
-			   iFFD.DocumentType="docbook";
-			   iFFD.Convertor=iFFD.DocumentType;
+			   iFFD.DocumentType = "docbook";
+			   iFFD.Convertor = iFFD.DocumentType;
+                           iFFD.Extension = "DCB";
 			   return(3);
 			   }
 		       break;
@@ -803,8 +814,9 @@ static int DetectOLEStream(FILE *stream, FFormatDetect & iFFD)
 {
 DWORD Magic1,Magic2;
 
-  iFFD.DocumentType="OLE Stream (might be Word or encapsulated WP)";
-  iFFD.Convertor="OLE Stream";
+  iFFD.DocumentType = "OLE Stream (might be Word or encapsulated WP)";
+  iFFD.Convertor = "OLE Stream";
+  iFFD.Extension = "OLE";
   iFFD.Probability=-1;
 
   Rd_dword(stream, &Magic1);
@@ -823,8 +835,8 @@ static int DetectPNG(FILE *stream, FFormatDetect & iFFD)
 static const unsigned char PngSignature[8] = {0x89,'P','N','G',0x0D, 0x0A, 0x1A, 0x0A};
 char buffer[sizeof(PngSignature)];
 
-  iFFD.DocumentType="PNG image";
-  iFFD.Convertor="PNG";
+  iFFD.DocumentType = "PNG image";
+  iFFD.Extension = iFFD.Convertor = "PNG";
   iFFD.Probability=-1;
   iFFD.DocVersion=0;
 
@@ -850,24 +862,25 @@ BYTE Magic2,Check3;
   Check3=fgetc(stream);
   fseek(stream,-4,SEEK_CUR);
 
+  iFFD.Extension = "TXT";
   if(Magic1 == 0xFEFF)	//This is an unicode text document
 	{
-	iFFD.DocumentType="Unicode Text";
-	iFFD.Convertor="UNICODE";
+	iFFD.DocumentType = "Unicode Text";
+	iFFD.Convertor = "UNICODE";
 	return 1;
 	}
   if(Magic1 == 0xFFFE)	//This is an unicode text document in High Endian
 	{
 	if(Magic2==0 && Check3==0) return 0;  //looks strange
 
-	iFFD.DocumentType="Unicode High Endian Text";
-	iFFD.Convertor="HIENDUNICODE";
+	iFFD.DocumentType = "Unicode High Endian Text";
+	iFFD.Convertor = "HIENDUNICODE";
 	return 1;
 	}
   if(Magic1==0xBBEF && Magic2==0xBF)	//This is an unicode text document in UTF8
 	{
-	iFFD.DocumentType="Unicode UTF8 Text";
-	iFFD.Convertor="UTF8";
+	iFFD.DocumentType = "Unicode UTF8 Text";
+	iFFD.Convertor = "UTF8";
 	return 1;
 	}
   return 0;
@@ -884,7 +897,8 @@ DWORD CurrentPos;
 DWORD cbMac=0xFFFF;
 
   iFFD.DocumentType="Microsoft Word";
-  iFFD.Convertor="WORD";
+  iFFD.Convertor = "WORD";
+  iFFD.Extension = "DOC";
 
   iFFD.DocVersion = 0x800;
 
@@ -947,8 +961,9 @@ WORD filetype, ver, encryption, dmp3;
 DWORD StartDoc;
 DWORD FilePos;
 
-  iFFD.Convertor=NULL;
-  FilePos=ftell(wpd);
+  iFFD.Convertor = NULL;
+  iFFD.Extension = "WP";
+  FilePos = ftell(wpd);
   Rd_dword(wpd, &id);
   Rd_dword(wpd,&StartDoc);
   Rd_word(wpd, &filetype);
@@ -1020,7 +1035,7 @@ DWORD FilePos;
 
     if(filetype>>8 == 0x16)
 	{
-	iFFD.Convertor="WPG";
+	iFFD.Extension = iFFD.Convertor = "WPG";
 	iFFD.DocVersion=ver<<8;
 	if(encryption!=0) fprintf(err,_(EncryptionWarning));
 	}
@@ -1029,7 +1044,7 @@ DWORD FilePos;
 	{
         switch(ver & 0xFF)
           {
-	  case 1: fprintf(err,_("\nWarning: Unknown MAC WP version 0x%x. Guessing WP%d.%d. EXPECT ERRORS!!! Please report."),ver,ver&0xFF,ver>>8);
+	  case 1: fprintf(err,_("\nWarning: Unknown MAC WP version 0x%x. Guessing WP%u.%u. EXPECT ERRORS!!! Please report."),ver,ver&0xFF,ver>>8);
           case 2: 	/*   version == MAC 2 -> WP 3.x  */
           case 3: 	/*   version == MAC 3 -> WP 3.x  */
           case 4: 	/*   version == MAC 4 -> WP MAC4.x  */
@@ -1037,7 +1052,8 @@ DWORD FilePos;
 	     iFFD.DocumentStart = StartDoc;
              if(encryption!=0)
 		{
-		if(encryption!=0) fprintf(err,_(EncryptionWarning));
+		if(Verbosing>=1 && err!=NULL) 
+		    fprintf(err,_(EncryptionWarning));
 		iFFD.Convertor="WP3.x encrypted";
 		}
              else
@@ -1068,7 +1084,7 @@ DWORD FilePos;
 		  break;
 	   default:iFFD.DocVersion=0x500l + (WORD)(ver>>8) + 256*(ver & 0xFF);
 		   iFFD.Convertor="WP6.x";	/* Place here -FORCE warning */
-		   fprintf(err,_("\nWarning: Unknown WP version 0x%x. Guessing WP%d.%d. EXPECT ERRORS!!! Please report."),ver,iFFD.DocVersion>>8,iFFD.DocVersion & 0xFF);
+		   fprintf(err,_("\nWarning: Unknown WP version 0x%x. Guessing WP%u.%u. EXPECT ERRORS!!! Please report."),ver,iFFD.DocVersion>>8,iFFD.DocVersion & 0xFF);
 		   encryption=0;
 	   }
 
@@ -1165,7 +1181,8 @@ unsigned CharNo;
 int DetectWordPerfect4(FILE *wpd, FFormatDetect & iFFD)
 {
   iFFD.DocumentType="Word Perfect";
-  iFFD.Convertor="WP4.x";
+  iFFD.Convertor = "WP4.x";
+  iFFD.Extension = "WP";
   iFFD.DocVersion=0x400;
 
   return DetectWP1and4(wpd, ObjWP4SizesC0);
@@ -1176,6 +1193,7 @@ int DetectWordPerfect1(FILE *wpd, FFormatDetect & iFFD)
 {
   iFFD.DocumentType="Word Perfect";
   iFFD.Convertor="WP1.x";
+  iFFD.Extension = "WP";
   iFFD.DocVersion=0x100;
 
   return DetectWP1and4(wpd, ObjWP1SizesC0);
@@ -1377,6 +1395,7 @@ static const PkgItem PkgList[] =
         {"PiFont",&pifont, N_("Use additional package pifont.")},
         {"rsfs",&Rsfs,NULL},
         {"Rsfs",&Rsfs, N_("Use mathrsfs with special characters.")},
+        {"scalerel",&Scalerel, N_("Use additional package scalerel.")},
         {"tipa",&TIPA,NULL},
         {"TIPA",&TIPA, N_("Use TIPA.sty for IPA and other phonetic characters")},
         {"ulem",&Ulem,NULL},
@@ -1655,107 +1674,121 @@ int p;
 	name += 2;
 
 		// check list of packages.
-        if(name[0]=='-')		// -no-
+        if(*name=='-')		// -no-
           {
-          int i;
-          for(i=0; i<sizeof(PkgList)/sizeof(PkgItem); i++)
+          name++;
+
             {
-            if(!strcmp(name+1, PkgList[i].Name)) break;
+            int i;
+            for(i=0; i<sizeof(PkgList)/sizeof(PkgItem); i++)
+              {
+              if(!strcmp(name+1, PkgList[i].Name)) break;
+              }
+            if(i<sizeof(PkgList)/sizeof(PkgItem))
+              {
+              *(PkgList[i].Flag) = -1;		// disable package
+              continue;
+              }
             }
-          if(i<sizeof(PkgList)/sizeof(PkgItem))
+             
+
+	  if(!strncmp(name, "lang",4))
             {
-            *(PkgList[i].Flag) = -1;		// disable package
+	    if(*name=='-') name++;	    	// -no-lang-
+
+	    for(int lng=0; lng<sizeof(LangTable)/sizeof(LangItem); lng++)
+	      {
+              if(!strcmp(name,LangTable[lng].LangDesc))
+	        {
+	        LangTable[lng].UseLang = -1;
+	        break;
+	        }
+	      }               
+            continue;
+	    }
+
+          if(!strcmp(name, "interactive"))
+            {
+            Interactive = false;
             continue;
             }
-          }
+          }	// end of -no-
 
-
-	if(!strncmp(name, "-lang",5))		// -no-lang-
-          {
-	  if(*name=='-') name++;	    
-
-	  for(int lng=0;lng<sizeof(LangTable)/sizeof(LangItem);lng++)
-	    {
-            if(!strcmp(name,LangTable[lng].LangDesc))
+        if(!strcmp(name, "clobber"))		// accepts -no-clobber and -noclobber
+              {
+              Noclobber = true;
+              continue;
+              }
+	if(!strcmp(name, "erase-strip")||!strcmp(name, "erasestrip")) /*-no-erase-strip*/
 	      {
-	      LangTable[lng].UseLang = -1;
-	      break;
+	      EraseStrip = false;
+	      continue;
 	      }
-	    }      
-         
-	  }	
-	else
-          {
-	  if(*name=='-') name++;
-
-          if(!strcmp(name, "clobber"))
-                {
-                Noclobber = true;
-                continue;
-                }
-	  if(!strcmp(name, "erase-strip")||!strcmp(name, "erasestrip")) /*-no-erase-strip*/
-		{
-		EraseStrip = false;
-		continue;
-		}
-          if(!strcmp(name, "extract-images")||!strcmp(name, "extractImages")||!strcmp(name, "extractimages")) /*-no-extract-WPG*/
+        if(!strcmp(name, "extract-images")||!strcmp(name, "extractImages")||!strcmp(name, "extractimages")) /*-no-extract-WPG*/
 		{
 		SaveWPG = -1;
 		continue;
 		}
-	  if(!strcmp(name, "extract-WPG")||!strcmp(name, "extractWPG")||!strcmp(name, "extractwpg")) /*-no-extract-WPG*/
+	if(!strcmp(name, "extract-WPG")||!strcmp(name, "extractWPG")||!strcmp(name, "extractwpg")) /*-no-extract-WPG*/
 		{
                 if(err!=NULL)
                   fprintf(err,"\n-no-extract-WPG is deprecated, use -no-extract-images instead.");
 		SaveWPG = -1;
 		continue;
 		}
-	  if(!strcmp(name, "fix-spaces")||!strcmp(name, "fixspaces")) /*-no-fix-spaces*/
+	if(!strcmp(name, "fix-spaces")||!strcmp(name, "fixspaces")) /*-no-fix-spaces*/
 		{
 		FixSpaces = false;
 		continue;
 		}
-	  if(!strcmp(name, "columns")||!strcmp(name, "Columns")) /*-no-columns*/
+	if(!strcmp(name, "columns")||!strcmp(name, "Columns")) /*-no-columns*/
 		{
 		Columns = -1;
 		continue;
 		}
-	  if(!strcmp(name, "inputPS")||!strcmp(name, "InputPS")) /*-no-inputPS*/
+	if(!strcmp(name, "inputPS")||!strcmp(name, "InputPS")) /*-no-inputPS*/
 		{
 		InputPS = -1;
 		continue;
-		}	
-	  if(!strcmp(name, "NFSS")||!strcmp(name, "nfss")) /*-no-NFSS*/
+		}
+#ifdef _REENTRANT
+	if(!strcmp(name, "jobs")||!strcmp(name, "Jobs")) /*-no-columns*/
+		{
+		JobCount = 0;
+		continue;
+		}
+#endif
+	if(!strcmp(name, "NFSS")||!strcmp(name, "nfss")) /*-no-NFSS*/
 		{
 		NFSS = false;
 		continue;
 		}
-	  if(!strcmp(name, "optimize-section")||!strcmp(name, "optimizesection")||!strcmp(name, "Optimize-Section"))
+	if(!strcmp(name, "optimize-section")||!strcmp(name, "optimizesection")||!strcmp(name, "Optimize-Section"))
 		{
 		OptimizeSection = false;
 		continue;
 		}
-	  if (!strcmp(name, "safemode")||!strcmp(name, "safe-mode")) /*-no-safemode*/
+	if(!strcmp(name, "safemode")||!strcmp(name, "safe-mode")) /*-no-safemode*/
 		{
 		ExtendedCheck = false;
 		continue;
 		}
-	  if (!strcmp(name, "texchars")||!strcmp(name, "TeXchars")) /*-no-texchars*/
+	if(!strcmp(name, "texchars")||!strcmp(name, "TeXchars")) /*-no-texchars*/
 		{
 		TexChars = false;
 		continue;
 		}
-	  if (!strcmp(name, "textcomp")) /*-no-textcomp*/
+	if(!strcmp(name, "textcomp")) /*-no-textcomp*/
 		{
 		textcomp = -1;
 		continue;
 		}
-	  if (!strcmp(name, "undoredo") || !strcmp(name, "UndoRedo")) /*-no-undoredo*/
+	if(!strcmp(name, "undoredo") || !strcmp(name, "UndoRedo")) /*-no-undoredo*/
 		{
 		UndoRedo = false;
 		continue;
 		}
-	  }	// enf of "-no-lang" else branch
+	  	// enf of "-no-lang" else branch
 
 
 	name = ArgName + 1;
@@ -1844,7 +1877,7 @@ int p;
 		}
       if(!strcmp(name, "v"))
 		{
-		fprintf(err,_("wp2latex %s   %s\n  modules:("),version,versiondate);
+		fprintf(err,_("wp2latex %s   %s\n  modules:("), VersionWP2L, VersionDate);
 		FFormatTranslator *Fx;
 		Fx = FFormatTranslator::first();
 		while(Fx!=NULL)
@@ -1914,7 +1947,8 @@ int p;
               log_filename = copy(wpd_filename,0,
 		      GetExtension(wpd_filename()) - wpd_filename()) + ".log";              
               }
-	    GetLogFileName(log_filename);
+            if(Interactive)
+	        GetLogFileName(log_filename);
 	    }
 	  continue;
 	  }
