@@ -22,19 +22,21 @@
 #include <stdlib.h>
 #include <math.h>
 
-//Non standard include files
-#include <common.h>
-#include <stringa.h>
-#include <lists.h>
-#include "wp2latex.h"
+#include "common.h"	// This header ensures that __UNIX__ is properly defined.
+
+#if defined(__UNIX__) || defined(__DJGPP__) || defined(_MSC_VER) 
+ #define SIGNALS_SUPPORTED
+ #include <signal.h>
+#endif
 
 #if defined(__UNIX__) || defined(__DJGPP__)
  #include <unistd.h>
 #endif
 
-#if defined(__UNIX__) || defined(__DJGPP__) || defined(_MSC_VER) 
- #include <signal.h>
-#endif
+//Non standard include files
+#include "stringa.h"
+#include <lists.h>
+#include "wp2latex.h"
 
 #ifdef _MSC_VER
  #include <locale.h>
@@ -43,7 +45,7 @@
 #ifdef _REENTRANT 
  #include "jobs/jobs.h"
  #ifndef INFINITE
-  #define INFINITE            0xffffffffL
+  #define INFINITE            0x7fffffffL
  #endif
 #endif
 
@@ -78,8 +80,7 @@ void Percentor::Actualise(long NewValue)
 	{
 	CurrentPercent=NewValue;
 	Show();
-	}
-		
+	}		
 }
 
 
@@ -149,8 +150,8 @@ void StrGets(string & s)
 {
 char c;
 
-  c=0;
-  s="";
+  c = 0;
+  s = "";
   while(c!='\n')
       {
       c=fgetc(stdin);
@@ -209,7 +210,7 @@ void RunError(int ExitCode, const char *s, const void *arg)
 	       break;
     case 0x4:  fprintf(err,_("\nError: File %s can not be created."),chk(s));
 	       break;
-    case 0x6:  fprintf(err,_("Error: Temporary file %s cannot be created!\n"),s);
+    case 0x6:  fprintf(err,_("Error: Temporary file %s cannot be created!\n"),chk(s));
 	       break;
     case 0x10: fprintf(err,_("Info: Program was aborted.\n"));
 	       break;
@@ -229,7 +230,7 @@ void RunError(int ExitCode, const char *s, const void *arg)
 		 const FFormatDetect *FFD = (const FFormatDetect *)arg;
 		 fprintf(err,"(%s",FFD->DocumentType);
 		 if(FFD->DocVersion!=0)
-		   fprintf(err," r%d.%d",FFD->DocVersion>>8,FFD->DocVersion & 0xFF);
+		   fprintf(err," r%u.%u", FFD->DocVersion>>8, FFD->DocVersion & 0xFF);
 		 fprintf(err,")");
 		 }
 	       fprintf(err,"!\n");
@@ -240,9 +241,8 @@ void RunError(int ExitCode, const char *s, const void *arg)
 	       break;
     }
 
-
-  if (wpd != NULL)   fclose(wpd);     	  wpd = NULL;
-  if (strip != NULL)
+  if(wpd != NULL)   fclose(wpd);     	  wpd = NULL;
+  if(strip != NULL)
   	{
 	fclose(strip);			  strip = NULL;
 #ifndef DEBUG
@@ -314,12 +314,12 @@ void Heading(void)
 {
 static bool printed=false;
  if(printed) return;
- printed=true;
+ printed = true;
 
  if(Verbosing < 1) return;
  printf(_("\n>>>WP2LaTeX<<< Conversion program: From Wordperfect to LaTeX Version %s \n"
 	  "  (c) Made by J.Fojtik  ---- Date : 1996-2020\n"),
-	    version);
+	    VersionWP2L);
 }
 
 
@@ -480,7 +480,7 @@ int p;
 
   if (latex_filename == "")
       {
-      if (Verbosing >= 1 || wpd_filename == "")
+      if(Interactive && (Verbosing>=1 || wpd_filename==""))
          {
 	 printf(_("\nWordPerfect-filename [%s]: "), chk(wpd_filename()));
 	 StrGets(StrName);
@@ -506,12 +506,12 @@ int p;
     if(p<=0) p=length(wpd_filename); //no extension
 
 
-    if (latex_filename == "")
+    if(latex_filename == "")
           {
 	  latex_filename=copy(wpd_filename,0,p) + ".tex";
 	  }
 
-    if (Verbosing >= 1 || latex_filename == "")
+    if(Interactive && (Verbosing >= 1 || latex_filename == ""))
        {
        printf(_("LaTeX-filename [%s]: "), chk(latex_filename()));
        StrGets(StrName);
@@ -549,8 +549,8 @@ static void CheckOptions(void)
 }
 
 
-#if (defined(__UNIX__)||defined(__DJGPP__)||defined(_MSC_VER)) & !defined(DEBUG)
-/* This is a main signal handler */
+#if defined(SIGNALS_SUPPORTED) & !defined(DEBUG)
+/** This is a main signal handler */
 void sig_int(int signo)
 {
 static int sigcount=0;
@@ -571,6 +571,19 @@ static void HookSignals(void)
 #endif
 
 
+void AskUser(void)
+{
+ if(!Interactive)
+   {
+   fprintf(err,_("\nFatal: Preventing to run defective code."));
+   abort();
+   }
+ fprintf(err,_("\nContinue anyway? y/N "));
+ if(toupper(getchar())!='Y') abort();
+ getchar();			//Get rid of buffered Enter key.
+}
+
+
 /** This procedure checks sizes of standard variables. */
 static void CheckCompiler(void)
 {
@@ -581,9 +594,7 @@ char str[20];
 	Heading();
 	fprintf(err,_("\nFatal: Code was badly compiled. sizeof(__u8)=%d sizeof(__u16)=%d sizeof(__u32)=%d\nCheck typedfs.h !!!!!"),
            (int)sizeof(__u8), (int)sizeof(__u16), (int)sizeof(__u32));	
-	fprintf(err,_("\nContinue anyway? y/N "));
-	if(toupper(getchar())!='Y') abort();
-	getchar();			//Get rid of buffered Enter key.
+        AskUser();
 	}
   sprintf(str,"%4.3f",1234.567);
   if(strcmp(str,"1234.567"))
@@ -598,10 +609,19 @@ char str[20];
     Heading();
     fprintf(err,_("\nFatal: Problem inside locale conversion of doubles '%s' expected '%s'!"),
        str,"1234.567");
-    fprintf(err,_("\nContinue anyway? y/N "));
-    if(toupper(getchar())!='Y') abort();
-    getchar();				//Get rid of buffered Enter key.
+    AskUser();
     }
+ else
+   {
+   string s;
+   s.printf("%4.3f",2345.678);
+   if(s!="2345.678")
+     { 
+     Heading();
+     fprintf(err,_("\nFatal: Printing to string does not work, obtained '%s' expected '%s'!"), s(), "2345.678");
+     AskUser();
+     }
+   }
 }
 
 
@@ -628,7 +648,7 @@ TconvertedPass1 *cqPass1;
   UnknownCharacters = UnknownObjects = CorruptedObjects = 0;
   cqPass1 = NULL;
 
-#if (defined(__UNIX__)||defined(__DJGPP__)||defined(_MSC_VER)) & !defined(DEBUG)
+#if defined(SIGNALS_SUPPORTED) & !defined(DEBUG)
   HookSignals();
 #endif
 
@@ -659,7 +679,7 @@ TconvertedPass1 *cqPass1;
     StyleOpCl = StyleOpCl2;
 
 #ifdef _REENTRANT
-  InitJobs(-1);
+  InitJobs(JobCount);
 #endif
 
   if((wpd = fopen(wpd_filename, "rb")) == NULL)
@@ -730,10 +750,10 @@ NoModule:  fprintf(err,_("\nType \"wp2latex -v\" to see a list of available modu
         if(err)
 	  fprintf(err,_("\nWarning: Cannot open log file %s ..."),log_filename());
 	}
-      else fprintf(logg,_("%% log file for converted file %s, %s ver %d.%d\n"
+      else fprintf(logg,_("%% log file for converted file %s, %s ver %u.%u\n"
 			  "%% conversion program WP2LaTeX v. %s from %s.\n"),
 			   wpd_filename(), chk(FilForD.DocumentType), FilForD.DocVersion>>8, FilForD.DocVersion & 0xFF,
-                           version, versiondate);
+                           VersionWP2L, VersionDate);
       }
     }
 
@@ -840,7 +860,14 @@ EmptyInput:
   if(JobBase::JobCount>0)
     {
     printf(_("Note: There are some asynchronous jobs that are still running."));
-    FinishJobs(INFINITE);	// Wait to finish them.
+    printf("--------");
+    int SecCounter=0;
+    while(JobBase::JobCount>0)
+    {
+      printf("\b\b\b\b\b\b\b\b%2dj%4ds", JobBase::JobCount, SecCounter++);
+      FinishJobs(1000);	// Wait to finish them.
+    }
+    printf(_("\nNote: All jobs finished now."));
     }
   EndJobs();
 #endif
